Package: RITAN
Type: Package
Title: Rapid Integration of Term Annotation and Network resources
Version: 1.34.0
Authors@R: c(person("Michael", "Zimmermann", role = c("aut", "cre"),
                    email = "mtzimmermann@mcw.edu"))
Description: Tools for comprehensive gene set enrichment and extraction of
    multi-resource high confidence subnetworks. RITAN facilitates bioinformatic
    tasks for enabling network biology research.
LazyData: TRUE
Depends: R (>= 4.0),
Imports: graphics, methods, stats, utils, grid, gridExtra, reshape2,
        gplots, ggplot2, plotrix, RColorBrewer, STRINGdb, MCL,
        dynamicTreeCut, gsubfn, hash, png, sqldf, igraph, BgeeDB,
        knitr, RITANdata, GenomicFeatures, ensembldb, AnnotationFilter,
        EnsDb.Hsapiens.v86
VignetteBuilder: knitr
Collate: 'lib_enrichment.R' 'lib_network.R'
        'interconnectivity_functions.R'
RoxygenNote: 7.3.2
Suggests: rmarkdown, BgeeDB
License: file LICENSE
biocViews: QualityControl, Network, NetworkEnrichment,
        NetworkInference, GeneSetEnrichment, FunctionalGenomics,
        GraphAndNetwork
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/RITAN
git_branch: RELEASE_3_22
git_last_commit: 0863028
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 06:06:20 UTC; biocbuild
Author: Michael Zimmermann [aut, cre]
Maintainer: Michael Zimmermann <mtzimmermann@mcw.edu>

# Generated by roxygen2: do not edit by hand

S3method(plot,term_enrichment)
S3method(plot,term_enrichment_by_subset)
S3method(summary,term_enrichment)
S3method(summary,term_enrichment_by_subset)
export(as.graph)
export(check_any_net_input)
export(check_net_input)
export(enrichment_symbols)
export(geneset_overlap)
export(icon_test)
export(load_geneset_symbols)
export(network_overlap)
export(readGMT)
export(readSIF)
export(resource_reduce)
export(show_active_genesets_hist)
export(term_enrichment)
export(term_enrichment_by_subset)
export(writeGMT)
export(write_simple_table)
import(AnnotationFilter)
import(EnsDb.Hsapiens.v86)
import(GenomicFeatures)
import(MCL)
import(RColorBrewer)
import(RITANdata)
import(STRINGdb)
import(dynamicTreeCut)
import(ensembldb)
import(ggplot2)
import(gplots)
import(graphics)
import(grid)
import(gridExtra)
import(gsubfn)
import(igraph)
import(plotrix, except=c('plotCI'))
import(png)
import(reshape2)
import(sqldf)
import(stats, except=c('decompose','lowess','spectrum'))
import(utils)
importFrom(methods,is)

# RITAN
RITAN is an R package for Rapid Integration of Term Annotation and Network resources.

RITAN brings together multiple resources, primarily to help answer two questions:

1. "What functions are achieved by this group of genes?"
2. "How do these genes work together to achieve that function?"

Specifically, RITAN has been written to:

1. Identify terms/pathways/genesets represented by (enriched within) the input list of genes
2. Identify network interactions among the genes within the input list (protein-protein, metabolic, and regulatory)

We have also built RITAN to facilitate research questions such as:

1. What terms/pathways share similar genes?
2. How do two resource's definitions of each pathway compare? List their common and unique sets.
3. From my genes of interest, use network resources to add genes that connect them together or are neighbors.
4. Annotate a combined list of genes with multiple functions contributed to.
6. Generate a heatmap for pathway/geneset -by- condition; for each timepoint, treatment, etc., show how term enrichment or pathway activity changes.
5. Make linking these results to other tools, such as Cytoscape, simpler via generating the focused and annotated subnetwork.

<hr>

RITAN is to be applied to an unranked list of gene symbols and we perform false-discivery adjustment across all resources used.
These and other topics are discussed in our publication:
<ul>
<li>Zimmermann, M.T. <i>et al.</i> RITAN: Rapid Integration of Term Annotation and Network Resources. <i>in review</i>.</li>
<li>"We encourage users to decide upon an analysis strategy prior to running their analysis, including consideration of which resources are most appropriate for their dataset/experiment, statistical significance thresholds, background geneset, etc. The ease with which RITAN facilitates multi-resource query may lead users to 'try one more test,' leading to an increased number of hypothesis tests made that may not be accounted for by multiple testing correction. To prevent this, we encourage users to 'include one more resource' - to add an additional resource to a single query in RITAN so that false-dicovery correction is appropriately maintained."</li>
</ul>

For additional examples of using RITAN, see our vignettes:

* `vignette('enrichment',package='RITAN')`
* `vignette('subnetworks',package='RITAN')`
* `vignette('choosing_resources',package='RITAN')`
* `vignette('resource_relationships',package='RITAN')`
* `vignette('multi_tissue_analysis',package='RITAN')`

<hr>

## For users interested in using RITAN, but who do not use R

We have made a standalone and self-contained executable available here:
https://mcw.box.com/s/bzmt12bgj2uygvcw6bk0hpg4p4w7nm2g
To use the app, download the .zip file, extract the zip file, and double-click the file, RITAN-electon.exe.

<!--
<p align="center">
  <img src="https://mcw.box.com/s/6y76mb22b06ovs3nxjocjz16al07o3up" width="350" type="image/pdf" title="Format for Group Comparison">
  <img src="https://mcw.box.com/s/zf3uqabksziu89uplabs2psnyonxfqwa" width="350" type="image/png" alt="Expected Result" title="Expected Result">
</p>
-->

<table align="center">
  <tr>
  <td> <a href="https://mcw.box.com/s/6y76mb22b06ovs3nxjocjz16al07o3up">Format for Group Comparison</a> </td>
  <td> <a href="https://mcw.box.com/s/zf3uqabksziu89uplabs2psnyonxfqwa">Expected Result</a> </td>
  </tr>  
</table>

The standalone app uses Chromium-electron and stand-alone R packaged together using node.js.
<br>
For making the combination of Shiny, RPortable, and electron, thank you to: https://github.com/ksasso/Electron_ShinyApp_Deployment

<hr>

# Bringing More Data into RITAN

I make an R script that has all my setup within. That way, I can simply `source('~/setup_RITAN.R')` and I'm ready to go. If you use RITAN frequently, I suggest writing the `source()` command in your `.Rprofile`. Below, are excerpts from my setup script.

To honor data redistribution policies, we provide certain annotation and network data in the RITANdata package. RITAN is most useful when many resources are organized for use. In my lab, we have many genesets, gene panels, pathway resources, gene regulatory networks, etc. organized and loaded into RITAN.

As a middle-ground, we provide here recommendations for downloading other resources and excerpts from my resrouce loading script for adding them to RITAN. Eventually, we plan to add a setup() function to RITAN that would automate this process. For now, we hope the below examples are helpful to users. We welcome feedback on the package and how to work more efficiently with existing open-source solutions.

```{r eval=FALSE, echo=TRUE, tidy=TRUE}
require(RITANdata)
require(RITAN)

## -------------------------------- -
## Set a couple of utility functions

apath <- '.' # annotation data path - script assumes annotation data are within a common path

csv_split <- function(x){ sort(unique(strsplit( gsub( ' ', '', x ) ,",")[[1]] )) }

read_dlm_as_sif <- function( f = NA, sep = "\t",
                             label_column = 1, gene_column = 2, ...){

  warning(sprintf('Reading delimited file into SIF format.\nAssuming labels are in column %d and genes are in column %d.\n', label_column, gene_column))

  d <- read.table( file=f, sep=sep, ... )
  l <- unique( d[ , label_column ] )
  sif <- list()
  for (n in l){
    i <- ( d[ , label_column ] == n )
    sif[[n]] <- d[ i, gene_column ] %>% unique() %>% sort()
  }

  return(sif)

}

## -------------------------------- -
## Add additional networks to RITAN

network_list$RegNet  <- readSIF( paste(apath, 'RegNetwork/human/human.source.sif', sep="/" ))
network_list$TRRUST  <- readSIF( paste(apath, 'TRRUST/trrust_rawdata.txt', sep="/" ))
network_list$IID     <- readSIF( sprintf('%s/IID/iid.human.2016-03.sif.gz',apath))
network_list$BioPlex <- readSIF( sprintf('%s/BioPlex/BioPlex.sif.gz',apath), et='BioPlex', score=3)
network_list$HPRD    <- readSIF( sprintf('%s/HPRD/HPRD_Release9_062910/BINARY_PROTEIN_PROTEIN_INTERACTIONS.txt', apath), p1=1, p2=4, et=7 )
network_list$BioGRID <- readSIF( sprintf('%s/BioGRID/BIOGRID-ORGANISM-Homo_sapiens-3.4.163.tab2.txt.gz', apath), p1=8, p2=9, et=12, score=19, quote='', skip=1, comment.char='' )
network_list$BioGRID$score <- as.numeric(network_list$BioGRID$score)

## -------------------------------- -
## Add additional genesets to RITAN

CPIC   <- read.csv( 'https://api.pharmgkb.org/v1/download/file/data/cpicPairs.csv', skip=1, header=TRUE, quote='', as.is=TRUE )
COSMIC <- read.table( sprintf("%s/COSMIC/cancer_gene_census_v83_GRCh38.csv",apath), sep=",", header=TRUE, as.is=TRUE)

geneset_list$MSigDB_C6     <- readGMT(sprintf('%s/genesets/c6.all.v6.1.symbols.gmt', apath))
geneset_list$CPIC_LevelABC <- sort(unique(CPIC$Gene[ ! grepl('D', CPIC$CPIC.Level) ])) %>% list(.)
geneset_list$HPO           <- read_dlm_as_sif( sprintf('%s/HPO/ALL_SOURCES_FREQUENT_FEATURES_phenotype_to_genes.2018-05-03.txt.gz', apath), label_column = 1, gene_column = 4, quote = '' )
geneset_list$monarch       <- read_dlm_as_sif( sprintf('%s/monarch/gene_disease.9606.tsv.gz', apath), label_column = 6, gene_column = 2, header = TRUE, quote = '' )
geneset_list$COSMIC        <- sort(unique(as.character( COSMIC$Gene.Symbol ))) %>% list(.)
geneset_list$ACMG56        <- sort(unique(strsplit("BRCA1 BRCA2 TP53 STK11 MLH1 MSH2 MSH6 PMS2 APC MUTYH VHL MEN1 RET PTEN RB1 SDHD SDHAF2 SDHC SDHB TSC1 TSC2 WT1 NF2 COL3A1 FBN1 TGFBR1 TGFBR2 SMAD3 ACTA2 MYLK MYH11 MYBPC3 MYH7 TNNT2 TNNI3 TPM1 MYL3 ACTC1 PRKAG2 GLA MYL2 LMNA RYR2 PKP2 DSP DSC2 TMEM43 DSG2 KCNQ1 KCNH2 SCN5A LDLR APOB PCSK9 RYR1 CACNA1S", " ")[[1]])) %>% list(.)

```

if(getRversion() >= "2.15.1"){
  utils::globalVariables( names=c('all_net', 'network_list'), package='RITAN')
}

#' as.graph
#'
#'wrapper to convert a data.frame from RITAN an igraph graph object
#'
#' @param mat matrix or data frame describing a network
#' @param p1 [1] column of first interactor
#' @param p2 [3] column of second interactor
#' @param ... further options passed on to igraph::graph()
#'
#' @return igraph object
#' @export
#'
#' @examples
#' \dontrun{
#' G <- as.graph(network_list$PID)
#' }
as.graph <- function( mat, p1 = 1, p2 = 3, ... ){
  # e.g. as.matrix( network_list$PID[, c(1,3)] ) %>% t(.) %>% c(.) %>% graph(.)
  graph( c(t( as.matrix(mat[, c(p1,p2)]) )), ... )
}


#' write_simple_table
#'
#'This is a simple wrapper around "write.table" that writes a tab-delimited table with column names, no quoting, and no row names.
#'
#' @param d R data object
#' @param f file path
#' @param ... further options passed on to write.table
#'
#' @return invisible (nothing is returned)
#' @export
#'
#' @examples
#' \dontrun{
#' simple wrapper around write.table for writing a tab-delimieted, no row names, tab-seperated file
#' }
write_simple_table <- function(d=NULL,f=NULL,...){
  write.table(d,f, quote = FALSE, sep="\t", col.names = TRUE, row.names = FALSE,...)
  invisible(NULL)
}

### ---------------------------------------------------------------- -
### Get the network to use
### ---------------------------------------------------------------- -
#' readSIF
#'
#'This function reads a data table into R; the data table describes network interactions. It is named for the Simple Interaction Format (SIF), but can read any data table if the users identifies which columns contain the pertinent data (see below).
#'
#'The SIF file format is a 3-column format, with an optional 4th column:
#'<entity-1><tab><edge-type><tab><entity-2><tab><score>
#'
#'Entities may be genes, proteins, metabolites, etc. The edge type typically conveys the type of relationship that exists between the two entities, such as physical interaciton, phosphorylation, or activation.
#'
#' @param file location of file
#' @param header indicator of presense of header on file
#' @param sep file delimiter - used by read.table()
#' @param as.is logical (default TRUE)
#' @param p1 Column number for the 1st entity. Default = 1.
#' @param p2 Column number for the 2nd entity. Default = 2.
#' @param et Column number for the edge type. Default = 3. Optionally, it may be a string label to be used as the edge type for all interactions from the input file.
#' @param score Column number for edge scores or weights. Default = NA (no score read).
#' @param ... Other options to read.table().
#'
#' @return Returns a data.frame with 3 (or 4) columns of data.
#' @export
#'
#' @examples
#' # Make a simple example to show the SIF file format
#' s <- matrix(c('gene1','gene2','PPI',
#'               'gene1','gene3','Chip-Seq',
#'               'gene4','gene5','PPI'), ncol=3, byrow=TRUE)
#' \dontrun{
#' # Read a SIF file
#' write.table( s, "myFile.sif", sep='\t', col.names=FALSE, row.names=FALSE )
#' sif <- readSIF("myFile.sif")
#' }
readSIF <- function( file = NA, header = FALSE, sep="\t", as.is=TRUE,
         p1=1, p2=2, et=3, score=NA, ... ){
  
  if (all(is.na(file))){
    
    stop('Please provide a valid file:
         readSIF("myfile") where myfile is a tab-delimited file with 3 columns. Each row of the file describes an edge in a network. By default, the first and second columns are genes/proteins/etc and the third column describes the interaction type. The optional fourth column contains a score or weight for the edge.')
    return(invisible(NULL))
    
  }
  
  if ( (!( methods::is(et,'character') | methods::is(et,'numeric')) ) || any(is.na(et)) || (length(et) != 1) ){
    stop('The edge type "et" must be a column number or string label. If a string label, all edges from the indicated file will have edge type "et."')
  }
  
  tmp <- read.table( file, header=header, sep=sep, as.is=as.is, ... )
  if (is.numeric(et)){
    
    sif <- data.frame( p1 = tmp[,p1],
                       edge_type = tmp[,et],
                       p2 = tmp[,p2], stringsAsFactors=FALSE )
    
  } else {
    
    sif <- data.frame( p1 = tmp[,p1],
                       edge_type = et,
                       p2 = tmp[,p2], stringsAsFactors=FALSE )
    
  }
  
  if (!all(is.na(score))){
    sif$score <- tmp[,score]
  }

  return(sif)

}
#' check_net_input
#'
#'A Quality Control function. This function will compare an input list of genes to a network reference and report if each member of the input is present in the resource.
#'
#' @param set An input list of genes to check against a reference.
#' @param ref A reference of network data. See readSIF().
#' @param check4similar Logical flag. If TRUE, a case-insensitive grep will be used for name matching. For genes in families with many related members (e.g. ABC*, FAM*, etc.), this will not be ideal. We intend this option as a QC screening method to identify if case, punctuaiton, etc is causing fewer than expected matches.
#' @param entity1name The column name in "ref" of the first entity. Default = "p1."
#' @param entity2name The column name in "ref" of the second entity. Default = "p2."
#'
#' @return Character vector of "yes/no" indicating "within-ref/not"
#' @export
#'
#' @examples
#' ## Return a "yes/no" vector indicating if each gene in myGeneSet is annotated with any term in GO
#' ## If no match, this function can attempt to suggest closest matches (check4similar = TRUE)
#' library(RITANdata)
#' myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
#' yorn <- check_net_input( myGeneSet, network_list[["CCSB"]] )
#' print(yorn)
#' 
#' yorn <- check_net_input( myGeneSet, network_list[["PID"]] )
#' print(yorn)
#' 
#' ## See check_any_net_input() for efficiently checking across all resources.
check_net_input <- function( set, ref, check4similar = FALSE, 
                             entity1name = "p1", entity2name = "p1" ){
  
  q <- sapply( set, function(x){
    
    h1 <- grepl( sprintf('^%s$',x), ref[[ entity1name ]], ignore.case = FALSE, perl=TRUE )
    h2 <- grepl( sprintf('^%s$',x), ref[[ entity2name ]], ignore.case = FALSE, perl=TRUE )
    
    if (any(h1|h2)){
      
      return('yes')
      
    } else {

      if (check4similar){
        h1 <- grepl( sprintf('%s',x), ref[[ entity1name ]], ignore.case = TRUE, perl=TRUE )
        h2 <- grepl( sprintf('%s',x), ref[[ entity2name ]], ignore.case = TRUE, perl=TRUE )
        r <- NULL

        if (any(h1)){ r <-      ref[[ entity1name ]][h1] ; }
        if (any(h2)){ r <- c(r, ref[[ entity2name ]][h2]); }

        if (!all(is.null(r))){
          return( sprintf('Potential Match: %s', paste( unique(r),sep=',',collapse = ',')) )
        }
      }

      return('no')
      
    }
    
  })
  
  return(q)

}

#' check_any_net_input
#'
#'A Quality Control function. This function applies check_net_input() to all available resources (default).
#'
#' @param set An input list of genes to check against references.
#' @param resources The collection of network resources to check within.
#' 
#' @return Logical vector indicating if the genes in "set" are within ANY of the resources.
#' @export
#'
#' @examples
#' #' ## Check if genes in myGeneSet are annotated by any resource in "network_list" (default).
#' library(RITANdata)
#' myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
#' yorn <- check_any_net_input( myGeneSet )
#' print(yorn)
check_any_net_input <- function(set, resources = names(network_list) ){
  
  # if (all(resources %in% names(network_list))){
  # } else {
  #   #warning('STRING and HPRD are now implemented as package calls, rather than indexed in network_list. Thus, they are not accessible to check_net_input()... This is on our "to-do" list...')
  #   # if (x == 'STRING'){
  #   #   
  #   # } else if (x == 'HPRD'){
  #   #   
  #   # } else if (x == 'Biogrid'){
  #   #   
  #   # }
  # }
  
  o <- lapply( network_list[resources] , function(y){
      
      check_net_input( set, y, check4similar = FALSE )
      
    } )
  
  a <- do.call( cbind, o )
  y <- apply( a, 1, function(y){ any(y=='yes') } )
  
  return(y)
  
}


# ### ---------------------------------------------------------------- -
# ### Gene Network from shared GO term similarity
# ### ---------------------------------------------------------------- -
# # GOSimNET
# #
# # param g List of genes to generate symantic similarities among.
# # param symbols Logical flag for if "g" contains gene symbols or gene IDs.
# # param ont Which ontology within GO to use. Default is "BP" for Biologic Process.
# # param measure Distance measure to use. Defult is "Wang."
# # param mart BioMart mart object
# # param ... further arguments passed on to mgeneSim()
# # 
# # return The similarity matrix beteen all input pairs of "g"
# # 
# # 
# # examples
# # dontrun{
# # ## This is a wrapper around functions from the package "GOSemSim"
# # ## Example 1:
# # g <- c('BRCA1','BRCA2','GSTM1','PTEN','PIK3CA')
# # n <- GOSimNet( g )
# #
# # ## Example 2:
# # g <- c("835","5261","241","994")
# # n <- GOSimNet( g, symbols = FALSE )
# # }
# GOSimNet <- function( g = NA, symbols = TRUE, ont="BP", measure="Wang", mart = NULL, ... ){
# 
#     if ( (length(g)==1) & is.na(g) ){
#       stop('Please provide a list of genes, either as HGNC symbols or enterz IDs (set symbols=FALSE).')
#     }
# 
#     # if (!requireNamespace("BiocManager", quietly=TRUE))
    #     # install.packages("BiocManager")
#     # BiocManager::install("GOSemSim")
#     # BiocManager::install("org.Hs.eg.db")
#     require(GOSemSim)
#     ## If you use GOSemSim in published research, please cite: G Yu, F Li, Y Qin, X Bo, Y Wu, S Wang.
#     ##   GOSemSim: an R package for measuring semantic similarity among GO terms and gene products. Bioinformatcs 2010, 26(7):976-978.
# 
# 
#     if (symbols){
#       ## symbols given. Map to Entrez gene IDs
#       
#       require(biomaRt)
#       if (all(is.null(mart))){
#         ## get Mart, unless one was privided
#         mart <- useMart("ENSEMBL_MART_ENSEMBL", dataset="hsapiens_gene_ensembl", host="useast.ensembl.org")
#       }
#       
#       s <- g
#       to_get <- c('hgnc_symbol', 'entrezgene')
#       o <- getBM(attributes = to_get, filters = 'hgnc_symbol', values = s, mart = mart )
#       g <- as.character(unique(o$entrezgene))
#       
#     }
#   
#     m <- mgeneSim( genes = g, ont=ont, organism="human", measure=measure, verbose=FALSE, ...)
#     
#     if (symbols){
#       e <- as.character(o$entrezgene)
#       i <- match( e, g )
#       if(any( e[i] != g )){ stop('Name mismatch.') }
#       rownames(m) <- colnames(m) <- o$hgnc_symbol[i]
#     }
# 
#     return(m)
# 
#     # We  also  implemented clusterSim for  calculating  semantic  similarity  between  two  gene  clusters,  and mclusterSim for calculating semantic similarities among multiple gene clusters.
#     # GO  enrichment  analysis  can  be  supported  by  our  package clusterProfiler [10],  which  supports  hyper-geometric  test  and  Gene  Set  Enrichment  Analysis (GSEA).  Enrichment  results  across  different  gene clusters can be compared using compareCluster function.
# 
# }


### ---------------------------------------------------------------- -
### Get the network to use
### ---------------------------------------------------------------- -
#' network_overlap
#'
#' @param gene_list A list of genes to use. The function will identify edges across resources for or among these genes; identify the induced subnetwork around the gene_list.
#' @param resources Name of network resource(s) to use.
#' @param minStringScore If STRING is among the resources, only edges of at least the indicated score will be included.
#' @param minHumanNetScore If HumanNet is among the resources, only edges of at least the indicated score will be included.
#' @param minScore Same as above, but used for any other networks where "score" is provided
#' @param verbose If TRUE (default), the function will update the user on what it is doing and how many edges are identified for each resource.
#' @param dedup If TRUE (Default = TRUE), remove edges reported by multiple resources. The edge type will be a semi-colon delimited list of the resources that had reported the interaction.
#' @param directed_net Logical indicating if the network resources should be interpreted as directed. 
#' @param include_neighbors Logical to include 1st neighbors of "gene_list" (genes not in gene_list, but directly connected to them) in the induced subnetwork.
#' @param STRING_cache_directory A directory where STRING data files are cached to speed up subsequent queries; no need to re-download. If NA (the default), caches STRING data in your Rpackages directory. If "", uses a temporary directory that is cleared when the R-session closes.
#' @param STRING_species Species taxon ID (number) to use in searching STRING data. (Default = 9606)
#' @param STRING_version Version of the STRING database (Default = "10")
#'
#' @return Data table describing the induced subnetwork for "gene_list" across the requested resources.
#' @import STRINGdb igraph MCL dynamicTreeCut sqldf gsubfn
#' @export
#'
#' @examples
#' ## Get interactions among a list of genes from the PID: Pathway Interaction Database
#' require(RITANdata)
#' myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
#' sif <- network_overlap( myGeneSet, resources = 'PID')
#' print(sif)
#' 
#' \dontrun{
#' ## Get the PPI network induced by genes within myGeneSet
#' ## Use 4 separate resources, but trim STRING to only include more confident interactions
#' sif <- network_overlap( myGeneSet, c('dPPI','PID','CCSB','STRING'), minStringScore = 500 )
#' }
network_overlap <- function( gene_list = NA, resources = c('PID','TFe','dPPI','CCSB','STRING'),
                    
                    # 7.8% have a score >= 700
                    minStringScore = 700,
                    
                    # 0.5 ==> 84.8%, 1.0 ==> 44.7%, 1.5 ==> 25.8%, 2.0 ==> 12.9%, 2.5 ==> 6.6%, 3.0 ==> 3.0%
                    minHumanNetScore = 0.4, 
                    minScore       = 0,
                    
                    verbose        = TRUE,
                    dedup          = TRUE,
                    directed_net   = FALSE,
                    include_neighbors = FALSE,
                    
                    STRING_cache_directory = NA,
                    STRING_species = 9606,
                    STRING_version = "10"
                    
                    ) {

  return_full_network <- FALSE
  if ( all(is.na(gene_list)) ||
       (!methods::is(gene_list, 'character')) ||
       (length(gene_list) == 0) ){
    
    if (verbose){
      cat('** No input list provided.\n** The full edge list from requested networks will be returned.\n')
    }
    return_full_network <- TRUE
    net <- do.call( rbind, lapply( resources, function(x){
      network_list[[x]][, c('p1','edge_type','p2') ] }) )
    
    return( net )
    
  }
  
  
  ## 
  supportedNetworks <- unique(c(names(network_list), 'STRING' ))
  i <- (resources %in% supportedNetworks) | (file.exists(resources))
  if (any(!i)){
    w1 <- sprintf('  Unsupported Networks Requested: %s\n', paste(resources[!i], sep=',', collapse=',') )
    w2 <- sprintf('    Supported Networks Are: %s\n', paste(supportedNetworks, sep=',', collapse=',') )
    w  <- paste0( w1, w2 )
    warning( w )
  }

  if (verbose){
    cat(sprintf('\nGenerating %sdirected subnetwork...\n', ifelse( directed_net, '', 'un') ))
  }
  if (directed_net){ stop('directed networks not yet implemented.') }
  
  n0 <- length(unique(gene_list))
  
  ## -------------------------- -
  select_edges <- function( x, inc_nei = include_neighbors, gs = gene_list ){
    
    y <- network_list[[x]][, c('p1','edge_type','p2') ]
    
    if (inc_nei){
      i <- (y$p1 %in% gs) | (y$p2 %in% gs)
    } else {
      i <- (y$p1 %in% gs) & (y$p2 %in% gs)
    }
    
    if ("score" %in% names(network_list[[x]]) ){
      if (x == 'HumanNet'){
        i <- i & (network_list[[x]]$score >= minHumanNetScore)
      #} else if (x == 'STRING'){
      #  i <- i & (network_list[[x]]$score >= minStringScore)
      } else {
        i <- i & (network_list[[x]]$score >= minScore)
      }
    }
    
    return(y[i,])
    
  }
  
  ## -------------------------- -
  ## Get all interactions for resources in "network_list"
  NetInList <- resources[ resources %in% names(network_list) ]
  sif <- data.frame( p1=character(0), edge_type=character(0), p2=character(0), stringsAsFactors=FALSE)
  sif <- do.call( rbind, lapply( NetInList , select_edges ) )
  
  ## Add neighbors, if requested
  if (include_neighbors){
    
    ## The first round is really to define which nodes/genes to consider.
    ## This second round will get all the edges within the full list.
    ## Without two rounds, we would just have "stars" around the input genes
    ##   and we'd miss the edges among neighbors.
    gene_list2 <- unique(c( gene_list, sif$p1, sif$p2 ))
    sif <- do.call( rbind, lapply( NetInList , function(x){
        select_edges(x, inc_nei = FALSE, gs = gene_list2 )
      } ) )
    
  }
  
  ## Check if anything was included - this is to prevent a NULL return
  if (is.null(dim(sif))){
    sif <- data.frame( p1=character(0), edge_type=character(0), p2=character(0), stringsAsFactors=FALSE)
  }
  
  
  ## -------------------------- -
  ## Initiate loader for STRING data
  if ('STRING' %in% resources){
    if (requireNamespace("STRINGdb", quietly = TRUE)) {
      
      map.input.to.STRING <- function( genes = NULL, s. = s, removeUnmappedRows = TRUE ){

        if (methods::is(genes, 'character')){
          genes <- data.frame(gene = genes)
        } else if ( (methods::is(genes, 'data.frame')) && ('gene' %in% names(genes)) ){

        } else {
          stop('Input should be a character vector of gene names/ids or a data frame with a column named "gene"')
        }

        mapped <- s.$map( genes, "gene", removeUnmappedRows = removeUnmappedRows )

        return(mapped)

      }
      
      if (all( is.na(STRING_cache_directory) )){
        
        j <- sapply( .libPaths(), function(x){
          d <- list.dirs(x, recursive = FALSE)
          i <- grepl('STRINGdb', d)
          return(sum(i))
        })
        
        if (all(j==0)){
          stop('I could not find STRINGdb in your .libPaths() - please ensure that STRINGdb is installed and in .libPaths().')
        }
        
        p <- .libPaths()[ which.max(j) ]
        STRING_cache_directory <- paste( p, 'STRINGdb.cache', sep='/' )
        
        # check if both directories are writable
        if ( ( file.access(STRING_cache_directory,2) == -1 ) ||
             ( file.access(p,2) == 1 ) ){
          stop('You do not have write access to STRING_cache_directory. You have requested to use STRING. Please change STRING_cache_directory to a directory that you have write permission for.')
        }
        
        dir.create(file.path(p, 'STRINGdb.cache'), showWarnings = FALSE)
        
        if (verbose){
          
          if ( length(list.files(STRING_cache_directory)) > 0 ){
            cat(sprintf('\tUsing previously cached STRING data files.'), sep="\n")
          } else {
            cat(sprintf('\tWe have detected that STRINGdb is installed here: %s
  \tWe will cache data from STRING here: %s
  \tTo override this behavior, please specify a "STRING_cache_directory."
  \tTo use a temporary directory, set STRING_cache_directory=""',
                      p, STRING_cache_directory ), sep="\n")
          }
          
        }
        
      }
      
      s <- STRINGdb::STRINGdb$new( version = STRING_version,
                         species = STRING_species,
                         score_threshold = minStringScore, # STRINGdb default = 400
                         input_directory = STRING_cache_directory )
      invisible(s$get_graph()) # loads all of STRING into igraph object
      
      input.genes.mapped <- map.input.to.STRING( genes = gene_list, s. = s )
      g <- input.genes.mapped$STRING_id
      
      vertex.names  <- get.vertex.attribute(s$graph,'name')
      vertex.search <- g[ g %in% vertex.names ]
      
      if (include_neighbors){
        
        n <- unique(unlist(sapply( vertex.search, function(x){ s$get_neighbors(x) })))
        i <- s$get_interactions( unique(c(g,n)) )
        
        # mapping_file <- 'ftp://ftp.ebi.ac.uk/pub/databases/genenames/new/tsv/locus_groups/protein-coding_gene.txt'
        mapping_file <- 'ftp://ftp.ebi.ac.uk/pub/databases/genenames/new/tsv/hgnc_complete_set.txt'
        tmp <- read.table( mapping_file, sep="\t", header=TRUE, as.is=TRUE, quote="", comment.char = '' )
        all.genes <- unique( tmp[, 'symbol' ])
        all.map   <- map.input.to.STRING( genes = all.genes, s. = s )
        
      } else {
        
        i <- s$get_interactions(g)
        all.map <- input.genes.mapped
        
      }
      
      ## NOTE: Of the two METHODS below, #2 better utilizes BioConductor standards, but #1 mapped more ENSPs to gene symbols.
      ##       Thus, I have kept using #1 for now... and its assumption that the user can read a public ftp://
      
      ## -- METHOD 1 -- Translate output to symbol from current ensembl mapping
      m.f <- match( i$from, all.map$STRING_id )
      m.t <- match( i$to  , all.map$STRING_id )
      i$from.symbol <- all.map$gene[m.f]
      i$from.symbol[ is.na(i$from.symbol) ] <- i$from[ is.na(i$from.symbol) ]
      i$to.symbol   <- all.map$gene[m.t]
      i$to.symbol[   is.na(i$to.symbol  ) ] <- i$to[   is.na(i$to.symbol  ) ]
      
      ## -- METHOD 2 -- Get the mapping from ENSP to symbol
      # i$from.symbol <- mapIds(hgu95av2.db,
      #                         keys=as.character(sapply( i$from, function(x){ strsplit(x,'.',fixed=TRUE)[[1]][2]} )),
      #                         column="SYMBOL", keytype="ENSEMBLPROT", multiVals="first")
      # 
      # i$to.symbol   <- mapIds(hgu95av2.db,
      #                         keys=as.character(sapply( i$to,   function(x){ strsplit(x,'.',fixed=TRUE)[[1]][2]} )),
      #                         column="SYMBOL", keytype="ENSEMBLPROT", multiVals="first")
      
      ## Add STRING interactions to the output
      sif <- rbind( sif, data.frame( p1 = as.character( i$from.symbol ),
                                     edge_type = sprintf('STRING%d',
                                                         round(i$combined_score/100)*100),
                                     p2 = as.character( i$to.symbol ), stringsAsFactors=FALSE
                                     ))
      
    } else {
      stop('STRING was requested, but the required package STRINGdb was not found by requireNamespace...')
    }
    
  }
  
  ## -------------------------- -
  ## Initiate loader for HPRD data
  #if ('HPRD' %in% resources){
  #  if (requireNamespace("ProNet", quietly = TRUE)) {
  #    
  #    hprd <- ProNet::construction(db="HPRD", ID.type= c("Gene symbol"), species = ProNet_species )
  #    id.hprd.match <- match( gene_list, V(hprd)$name )
  #    v <- id.hprd.match[ !is.na(id.hprd.match) ]
  #    g.hprd <- induced.subgraph(hprd, v)
  #    
  #    if (include_neighbors){
  #      n <- unique(unlist(sapply( v, function(x){ as.numeric(neighbors(hprd,x)) })))
  #      g.hprd <- induced.subgraph(hprd, unique(c(v, as.numeric(n))))
  #    }
  #    
  #    i <- get.edgelist(g.hprd, names=TRUE)
  #    
  #    sif <- rbind( sif, data.frame( p1 = as.character( i[, 1]),
  #                                   edge_type = rep('HPRD', dim(i)[1] ),
  #                                   p2 = as.character( i[, 2] ), stringsAsFactors=FALSE
  #                                   ))
  #  } else {
  #    stop('HPRD was requested, but the required package ProNet was not found by requireNamespace...')
  #  }
  #  
  #}
  
  ## -------------------------- -
  ## Initiate loader for BioGrid data
  #if ('Biogrid' %in% resources){
  #  if (requireNamespace("ProNet", quietly = TRUE)) {
  #    
  #    biog <- ProNet::construction(db="Biogrid",ID.type= c("Gene symbol"), species = ProNet_species )
  #    
  #    id.biog.match <- match( gene_list, V(biog)$name )
  #    v <- id.biog.match[ !is.na(id.biog.match) ]
  #    g.biog <- induced.subgraph(biog, v)
  #    
  #    if (include_neighbors){
  #      n <- unique(unlist(sapply( v, function(x){ as.numeric(neighbors(biog,x)) })))
  #      g.biog <- induced.subgraph(biog, unique(c(v, as.numeric(n))))
  #    }
  #    
  #    i <- get.edgelist(g.biog, names=TRUE)
  #    
  #    sif <- rbind( sif, data.frame( p1 = as.character( i[, 1] ),
  #                                   edge_type = rep('Biogrid', dim(i)[1] ),
  #                                   p2 = as.character( i[, 2] ), stringsAsFactors=FALSE
  #                                   ))
  #    
  #  } else {
  #    stop('Biogrid was requested, but the required package ProNet was not found by requireNamespace...')
  #  }
  #  
  #}
  
  ## -------------------------- -
  ## Read in user-supplied SIF files (assumes SIF file format)
  if( any(file.exists(resources)) ){
    
    f <- which(file.exists(resources))
    for (fi in f){
      
      if (verbose){
        cat(sprintf('Reading SIF file: %s', resources[fi]), sep="\n" )
      }
      
      y <- readSIF( resources[fi] )
      
      if (include_neighbors){
        hit <- (y$p1 %in% gene_list) | (y$p2 %in% gene_list)
      } else {
        hit <- (y$p1 %in% gene_list) & (y$p2 %in% gene_list)
      }
      
      if (dim(y)[2] >= 4){
        hit <- hit & (y[,4] >= minScore)
      }
      
      if (verbose){
        cat(sprintf('\tidentified %d edges', sum(hit) ), sep="\n" )
      }
      
      if (any(hit)){
        sif <- rbind( sif, y[ hit, 1:3] )
      }
      
    }
    
  }
  
  ## -------------------------- -
  row_NA <- apply(sif,1,function(x){any(is.na(x))})
  if (any(row_NA)){
    warning(sprintf('\nWe have detected NAs in %d rows. These rows will be removed. They are most likely from ENSPs in STRING that do not map to a gene symbol... We are working to improve the mappings used to prevent these occurrences.\n', sum(row_NA)))
    sif <- sif[ !row_NA, ]
  }
  
  ## -------------------------- -
  if(dedup && (dim(sif)[1] > 0) ){
    
    if (verbose){
      cat('\nRemoving duplicate edges and self-loops...\n')
    }

    ## easiest reduction first: full duplicates
    sif <- unique(sif)
    
    ## remove self loops
    ## NOTE: this can "remove" nodes from the network if the self-loop is the only edge they have
    i <- sif$p1 == sif$p2
    if (any(i)){
      sif <- sif[ !i, ]
    }

    ## collapse duplicate edges
    if (directed_net){
      # unique and directed.
      tmp <- sif[,c(1,3)]
    } else {
      # need to sort genes within each row - this operation preserves row order
      tmp <- t(apply( sif, 1, function(x){ sort(x[c(1,3)]) } ))
    }
    
    ## ignore "source" for detecting duplicates
    i <- duplicated(tmp)
    
    if (any(i)){
      
      ## But, retain "source" labels in the output
      to_remove <- unique(tmp[i,])
      for ( j in 1:dim(to_remove)[1] ){
        k <- ( (sif$p1 == to_remove[j,1]) & (sif$p2 == to_remove[j,2]) ) |
             ( (sif$p1 == to_remove[j,2]) & (sif$p2 == to_remove[j,1]) )
        sif$edge_type[ k & (!i) ] <- paste( sort(unique( sif$edge_type[ k ] )), sep=';', collapse=';' )
      }
      
      sif <- sif[ !i, ]
      
    }

  }
  
  
  if (verbose){
    n <- unlist(sif[,c(1,3)])
    if (!return_full_network){
      cat(sprintf('Total induced subnetwork from %d genes has %d nodes and %d edges (%d unique).\n',
                  n0, length(unique(n)), dim(sif)[1], dim(unique(sif[,c(1,3)]))[1] ))
    } else {
      cat(sprintf('Total network has %d nodes and %d edges.\n', length(unique(n)), dim(sif)[1] ))
    }
  }
  
  ## Add attributes to the output so that we can always check what was actually run.
  attr(sif,'param') <- data.frame( 'resources' = resources,
                                   'minStringScore' = minStringScore,
                                   'minHumanNetScore' = minHumanNetScore,
                                   'minScore' = minScore,
                                   'dedup' = dedup,
                                   'directed_net' = directed_net,
                                   'include_neighbors' = include_neighbors,
                                   'STRING_cache_directory' = STRING_cache_directory,
                                   'STRING_species' = STRING_species,
                                   'STRING_version' = STRING_version
                                   )
  
  ## done
  return(sif)

}

## ----first_load, echo=TRUE, warning=FALSE, results='hide', message=FALSE------
library(RITANdata)
library(RITAN)

## ----quick_start, echo=TRUE, warning=FALSE------------------------------------
# We define a query using gene symbols
my_genes <- c('ABCA2','ACAT2','ACSS2','CD9','CPEB2','CTNNB1','FASN','LDLR','LPL','LSS')

# Now, you can rapidly get enrichment results by:
# e <- term_enrichment(my_genes)

# However, we will make two simplifications here, for expediency.
#  1) only look at two resources (of the many available)
#  2) use a cached background gene list of human protien coding genes
resources <- c("ReactomePathways", "MSigDB_Hallmarks")
e <- term_enrichment(my_genes, resources = resources, all_symbols = cached_coding_genes)
summary(e)

## ----echo_geneset_list_names, echo=TRUE---------------------------------------
names(geneset_list)

## ----echo_geneset_list_names2, echo=TRUE--------------------------------------
head(geneset_list$MSigDB_C7$GSE9988_LPS_VS_LOW_LPS_MONOCYTE_UP)

## ----make_study_selection, echo = TRUE----------------------------------------
selection <- grepl( 'GSE9988_(LOW_)*LPS_VS_.+UP', names(geneset_list$MSigDB_C7), perl=TRUE )
study_set <- geneset_list$MSigDB_C7[selection]
str(study_set)

## ----apply_enrichment_to_selection, echo=TRUE---------------------------------
e <- term_enrichment_by_subset( study_set, q_value_threshold = 1e-5, 
                                resources = resources,
                                all_symbols = cached_coding_genes )

## ----ep1, echo=TRUE, fig.width = 7, fig.height = 6----------------------------
plot( e, show_values = FALSE, label_size_y = 7, label_size_x = 7 )

## ----ep1_cap, echo=TRUE, fig.width = 7, fig.height = 6------------------------
plot( e, show_values = FALSE, label_size_y = 7, label_size_x = 7, cap=10 )

## ----ep1_data, echo=TRUE, fig.width = 7, fig.height = 6-----------------------
prmatrix( e[1:3,], collab = c('name','n.set',1:7) )

## ----make_ann_mat, echo=TRUE--------------------------------------------------
mat <- matrix(c("LPS","LPS","LPS","LPS","LOW_LPS","LOW_LPS","LOW_LPS",
                "LOW_LPS","LPS_AND_ANTI_TREM1","CTRL_TREATED",
                "VEHICLE_TREATED","ANTI_TREM1_AND_LPS","CTRL_TREATED","VEHICLE_TREATED"),
                nrow = 2, byrow = TRUE )
rownames(mat) <- c('Condition1','Condition2')
colnames(mat) <- sprintf('Sample%s', 1:7)
print(mat)

## ----show_ann_mat, echo=TRUE, fig.width = 7, fig.height = 8-------------------
plot( e, show_values = TRUE, label_size_y = 7, label_size_x = 7, cap=10, 
      annotation_matrix = mat, grid_line_color = 'black' )

## ----apply_enrichment_to_selection_n, echo=TRUE-------------------------------
n <- term_enrichment_by_subset( study_set, q_value_threshold = 1e-5,
                                resources = resources,
                                all_symbols = cached_coding_genes,
                                display_type = 'n', phred = FALSE )

## ----apply_enrichment_to_selection_n_plot, echo=TRUE, fig.width = 7, fig.height = 6----
plot( n, show_values = TRUE, label_size_y = 7, label_size_x = 7, cap = 20 )

## ----term_enrichment1, eval=FALSE---------------------------------------------
# data("vac1.day0vs31.de.genes")
# te <- term_enrichment( geneset = vac1.day0vs31.de.genes )

## ----term_enrichment2, eval=FALSE---------------------------------------------
# e <- term_enrichment( geneset = vac1.day0vs31.de.genes, verbose = TRUE,
#                       resources = c("Blood_Translation_Modules", "MSigDB_C7") )

## ----term_enrichment3, eval=FALSE---------------------------------------------
# e <- term_enrichment( geneset = vac1.day0vs31.de.genes, verbose = TRUE,
#                       resources = names(geneset_list) )

## ----term_enrichment_add_gmt, echo=TRUE---------------------------------------
gs  <- geneset_list$MSigDB_C7[['GSE6269_HEALTHY_VS_FLU_INF_PBMC_UP']]
gmt <- system.file("extdata", "curated_gene_disease_associations.gmt.gz", package="RITAN")

# -->> Not running here for brevity
# geneset_list$DisGeNet <- readGMT(gmt)
# str(head(geneset_list$DisGeNet))

## ----term_enrichment_provided_gmt, echo=TRUE----------------------------------
e2  <- term_enrichment( gs, resources = gmt, all_symbols = cached_coding_genes )
print( e2[1:3,-5] )

## ----term_enrichment_search_gmt, echo=TRUE------------------------------------
geneset_list$DisGeNet <- readGMT(gmt)
print( geneset_list$DisGeNet[['Influenza, Human']] )

## ----show_hist, echo=TRUE-----------------------------------------------------
show_active_genesets_hist()
length(active_genesets)

## ----LDL, echo=TRUE-----------------------------------------------------------
geneset_list$LDL = list( LDL_import = c('APOB','APOE','LDLR'), 
                         LDL_processing = c('HMGR','ACAT2','HMGCS1',
                                            'HMGCR','MVD','MVK',
                                            'PMVK','IDI1','IDI2') )

## ----LDLe, eval=FALSE---------------------------------------------------------
# e <- term_enrichment( gs, resources = c('GO','LDL') )

#' icon_test
#' 
#' "icon" is an abbreviation for the "interconnectivity" of a network or graph.
#' 
#' This function handles different inputs and directs them to the appropriate "icon" testing method.
#' Depending on the values given to "nodes1" and "nodes2," a different specific test is performed.
#' 
#' Note that the specific functions called make use of the "param" attribute of each input.
#' These parameters are populated by network_overlap() so that the permutation reflects the exact procedure that was done to generate "nodes1" and/or "nodes2."
#' 
#' @param nodes1 [NULL] the first network. See network_overlap().
#' @param nodes2 [NULL] the second network. See network_overlap().
#' @param s [100] the number of random permutations to make.
#' @param verbose [TRUE] Extent of text shown in the console.
#' @param ... Additional argumetns are passed on to the specific test performed
#' 
#' @export
#' @return metrics and significance of the network overlap
#' 
#' @examples
#' \dontrun{
#' icon_test( nodes1=n, s=10)
#' }
#' 
icon_test <- function( nodes1 = NULL, nodes2 = NULL, s=100, verbose=TRUE, ... ){
  
  stop('TO DO: update all tests to use parameters from network_overlap() results')
  param1 <- attr(nodes1,'param')
  
  test_mode <- 'dual_between'
  if (all(is.null(c(nodes1,nodes2)))){
    stop('input required')
  }
  #print(class(nodes1))

  o <- NULL
  if ( all(is.null(nodes2)) || all(is.na(nodes2)) ){

    if (methods::is(nodes1, 'character')){
      test_mode <- 'single_within'
      o <- icon_single_within( nodes1, s )
    } else if (methods::is(nodes1, 'list')){
      test_mode <- 'list_between'
      o <- icon_list_between( nodes1, s, verbose=verbose, ... )
    }

  } else {
    o <- icon_dual_between( nodes1, nodes2, s, verbose=verbose, ... )
  }

  o$test_mode <- test_mode
  return(o)

}

#' cov_undirected
#' function to show the un-directed coverabe between two nodes lists from two networks
#' @param this_nodes1 list of nodes for first network
#' @param this_nodes2 list of nodes for second network
#' @param this_net1 the first network
#' @param this_net2 the second network
cov_undirected <- function( this_nodes1, this_nodes2, this_net1, this_net2 ){
  cov12 <- ( this_nodes1 %in% this_net2$p1 ) | ( this_nodes1 %in% this_net2$p2 )
  cov21 <- ( this_nodes2 %in% this_net1$p1 ) | ( this_nodes2 %in% this_net1$p2 )
  gin <- unique(c( this_nodes1[ cov12 ], this_nodes2[ cov21 ] ))
  ag  <- unique(c( this_nodes1, this_nodes2 ))
  obs <- length( gin ) / length(ag)
  return(obs)
}

#' icon_single_within
#' interconnectivity score within a network
#' @param nodes the node labels to use
#' @param net the network to use
#' @param s [10] the number of repeated random draws to make
#' @param verbose [TRUE] if more verbose output should be shown
icon_single_within <- function( nodes = NULL, net = NULL, s=10, verbose=TRUE ){

  if (all(is.null(net))){
    stop('you must input a network to search within')
  }
  if (all(is.null(nodes))){
    stop('you must input a list of node labels to search for, within the network')
  }

  int_within <- (nodes %in% net$p1) | (nodes %in% net$p2)
  obs <- sum( int_within ) / length(nodes)

  ran <- rep(NA,s)
  if (verbose){ pb <- txtProgressBar( style = 3 ); }
  for ( k in 1:s ){
    if (verbose){ setTxtProgressBar(pb = pb, value = k/s) }

    r <- sample( all_symbols, length(nodes), replace=FALSE )
    n_ran <- all_net[ (all_net$p1 %in% r) & (all_net$p2 %in% r), ]
    i_ran <- (r %in% n_ran$p1) | (r %in% n_ran$p2)
    ran[k] <- sum( i_ran ) / length(r)
  }

  out <- list()
  out$obs <- obs
  out$ran <- ran
  out$p <- sum(ran) >= obs
  out$z <- (obs - median(ran)) / mad(ran, constant = 1)

  if (verbose){
    d <- density(ran)
    d$y <- d$y / sum(d$y)
    plot( 0, type='n', xlim=range(c(obs,ran)), ylim=range(d$y),
          main = sprintf('I = %.2f. Z = %.1f', obs, out$z ),
          xlab = 'Interconnectivity', ylab = 'Probability Density' )
    polygon(d, col = 'gray')
    abline( v=obs, col='red')
    box()
  }

  return(out)

}

icon_dual_between <- function( nodes1=NULL, nodes2=NULL, s,
                               verbose=TRUE, net1=NULL, net2=NULL ){

  if ( all(is.null(net1)) || all(is.null(net2)) ){
    stop('you must input 2 networks to search between')
  }
  
  if ( all(is.null(nodes1)) || all(is.null(nodes2)) ){
    stop('you must input two lists of node labels to search for, between the two networks')
  }
  
  obs <- cov_undirected( nodes1, nodes2, net1, net2 )
  
  ran <- rep(NA,s)
  if (verbose){ pb <- txtProgressBar( style = 3 ); }
  for ( k in 1:s ){
    if (verbose){ setTxtProgressBar(pb = pb, value = k/s) }
    r1 <- sample( all_symbols, length(nodes1), replace=FALSE )
    r2 <- sample( all_symbols, length(nodes2), replace=FALSE )
    nodes1_ran <- all_net[ (all_net$p1 %in% r1) | (all_net$p2 %in% r1), ]
    nodes2_ran <- all_net[ (all_net$p1 %in% r2) | (all_net$p2 %in% r2), ]
    ran[k] <- cov_undirected( r1, r2, nodes1_ran, nodes2_ran )
  }

  out <- list()
  out$obs <- obs
  out$ran <- ran
  out$p <- sum(ran) >= obs
  out$z <- (obs - median(ran)) / mad(ran, constant = 1)

  if (verbose){
    d <- density(ran)
    d$y <- d$y / sum(d$y)
    plot( 0, type='n', xlim=range(c(obs,ran)), ylim=range(d$y),
          main = sprintf('I = %.2f. Z = %.1f', obs, out$z ),
          xlab = 'Interconnectivity', ylab = 'Probability Density' )
    polygon(d, col = 'gray')
    abline( v=obs, col='red')
    box()
  }

  return(out)

}


icon_list_between <- function( n_list, s, verbose=TRUE, net_list = NULL ){

  preCompLists <- FALSE
  if ( all(!is.null(net_list)) & (length(n_list) == length(net_list)) ){
    preCompLists <- TRUE
  }
  #cat(sprintf('Using pre-compiled lists? %s', preCompLists))

  m <- length(n_list)
  z_mat <- array( 0, dim=c(m,m) )
  for (i in 1:m){
    if(verbose){
      cat(sprintf('\nCalculating interactions with input set %d.',i))
    }

    ## NOTE: the "within" lists shoudl not have neighbors, but the "between" should...
    #if(preCompLists){
    #  z_mat[i,i] <- icon_single_within( n_list[[i]], s=s, verbose = verbose,
    #                                    net_within = net_list[[i]] )$z
    #} else {
      z_mat[i,i] <- icon_single_within( n_list[[i]], s=s, verbose = verbose )$z
    #}

    if ( (i+1) <= m ){
    for (j in (i+1):m){

      if(preCompLists){
        z_mat[i,j] <- z_mat[j,i] <-
          icon_dual_between( n_list[[i]], n_list[[j]], s=s, verbose=verbose,
                             net_list[[i]], net_list[[j]] )$z
      } else {
        z_mat[i,j] <- z_mat[j,i] <-
          icon_dual_between( n_list[[i]], n_list[[j]], s=s, verbose=verbose )$z
      }

    }}
  }

  out <- list()
  out$z_mat <- z_mat
  return(out)

}

### TO IMPLEMENT
## - distributional test between random networks and a bootstrapped estimate from your data better than current interconnectivity test? (would just be a more confident estimate of observation)

# if(getRversion() >= "2.15.1"){
#   utils::globalVariables( names=c('active_genesets', 'all_symbols', 'geneset_list'), package='RITAN')
# }
resources.default = c("GO", "ReactomePathways", "KEGG_filtered_canonical_pathways", "MSigDB_Hallmarks")

### --------------------------------------------------------------- -
## Declare datasets
### --------------------------------------------------------------- -

#' This dataset is included as an example in the package:
#' 
#' @examples
#' \dontrun{
#'  #data("vac1.day0vs31.de.genes")
#'  te <- term_enrichment(geneset = vac1.day0vs31.de.genes)
#' }
#' 
#' @return differentially expressed genes at 31 days post-vaccination with vaccine1
#' @references \url{https://www.ncbi.nlm.nih.gov/pubmed/26755593}
"vac1.day0vs31.de.genes"

#' This dataset is included as an example in the package:
#' 
#' @examples
#' \dontrun{
#'  #data("vac1.day0vs56.de.genes")
#'  te <- term_enrichment(geneset = vac1.day0vs56.de.genes)
#' }
#' 
#' @return differentially expressed genes at 56 days post-vaccination with vaccine1
#' @references \url{https://www.ncbi.nlm.nih.gov/pubmed/26755593}
"vac1.day0vs56.de.genes"

#' This dataset is included as an example in the package:
#' 
#' @examples
#' \dontrun{
#'  #data("vac2.day0vs31.de.genes")
#'  te <- term_enrichment(geneset = vac2.day0vs31.de.genes)
#' }
#' 
#' @return differentially expressed genes at 31 days post-vaccination with vaccine2
#' @references \url{https://www.ncbi.nlm.nih.gov/pubmed/26755593}
"vac2.day0vs31.de.genes"

#' This dataset is included as an example in the package:
#' 
#' @examples
#' \dontrun{
#'  #data("vac2.day0vs56.de.genes")
#'  te <- term_enrichment(geneset = vac2.day0vs56.de.genes)
#' }
#' 
#' @return differentially expressed genes at 56 days post-vaccination with vaccine2
#' @references \url{https://www.ncbi.nlm.nih.gov/pubmed/26755593}
"vac2.day0vs56.de.genes"


## Enrichment functions - primary function is term_enrichment.
### --------------------------------------------------------------- -
#' readGMT
#'
#' Created for simplification of reading .gmt files into RITAN.
#'
#' @param f GMT file name. Please provide a full path if the file is not in the current working directory.
#'
#' @return A list() where the name of each entry is the term (first column of GMT file) and the value is a chr array of genes associated with the term.
#' @export
#'
#' @examples
#' # Make an example list() to show the GMT format
#' set <- list( term1=c('gene_name1','gene_name2'),
#'              term2=c('gene_name3','gene_name4','gene_name5') )
#' \dontrun{
#' # Write a GMT file for "set"
#' writeGMT( set, 'my_file.gmt' )
#' 
#' # Reading GMT files
#' geneset <- readGMT( 'my_file.gmt' )
#' 
#' # Additional GMT files are available from multiple sources.
#' #   We recommend:
#' #   http://software.broadinstitute.org/gsea/msigdb/
#' }
#' 
readGMT <- function( f = NA ){
  
  if(all(is.na(f))){
    stop('Please provide an input file:
         readGMT("myfile") where myfile is a tab-delimited file. Each row of the file describes a geneset. The first column is the name of the geneset. The second is the source for the geneset. Each further column lists genes within the geneset.')
    return(invisible(NULL))
  }
  
  sets <- list()
  raw  <- read.table( f, as.is=TRUE, sep='|', fill=TRUE, quote='' )$V1
  for ( i in 1:length(raw) ){
    tmp <- strsplit( raw[[i]], '\t' )[[1]]
    sets[[i]] <- sort(unique(as.character( setdiff( tmp[3:length(tmp)], '' ) )))
    names(sets)[i] <- tmp[1]
  }
  return(sets)
}

#' writeGMT
#'
#' Created for future use and simplification of writing .gmt files from the package.
#'
#' @param s list of gene sets in current R session. Each entry will become a row in the GMT file.
#' @param file file name to write to
#' @param link default is "". This is the second column of a GMT file and is usually a hyperlink or note about the origin of the term
#'
#' @return Nothing is returned. A file is written.
#' @export
#' 
#' @examples
#' # Make an example list() to show the GMT format
#' set <- list( term1=c('gene_name1','gene_name2'),
#'              term2=c('gene_name3','gene_name4','gene_name5') )
#' \dontrun{
#' # Write a GMT file for "set"
#' writeGMT( set, 'my_file.gmt')
#' }
writeGMT <- function( s, file = NA, link=rep('', length(s)) ){
  
  if(all(is.na(file))){
    stop('Please provide data and a file name:
         writeGMT(geneset_def, "myfile") where "geneset_def" is an object of list class that describes which genes are contained in a list of genesets.')
    return(invisible(NULL))
  }
  
  if (is.na(file)){ stop('no file name given') }
  if (!methods::is(s, 'list')){ stop('input should be a list of genesets') }
  cat('', file=file, append=FALSE)
  for (x in 1:length(s) ){
    cat( sprintf('%s\t%s\t%s\n', names(s)[x], link[x],
                 paste( unique(s[[x]]), sep='\t', collapse='\t' ) ),
         file=file, append=TRUE )
  }
  return(NULL)
}


#' load_geneset_symbols
#'
#' For most applications, this function is used internally by term_enrichment(). Users may call this function directly in some cases to force FDR adjustment to be across multiple resources. See Vignette for more details.
#'
#' load_geneset_symbols allows the user to specify an annotation resource (e.g. Gene Ontology terms)
#' to use in enrichment analysis. The expectation is that the annotation resource contains of at least
#' one set of genes in the form of a list. The RITAN package comes with 15 pre-loaded annotation
#' resources. The default active annotation resources are GO, ReactomePathways,
#' KEGG_filtered_canonical_pathways, and MSigDB_Hallmarks.
#'
#' The result of calling this function is to set the variable "active_genesets" which will be used by further functions.
#'
#' @param gmt Either 1) name of pre-loaded resource (i.e. names(geneset_list)) or 2) gmt file containing annotation resources for enrichment annotation
#' @param gmt_dir location of gmt file named in gmt parameter
#' @param verbose print results to screen
#'
#' @return R list object named active_genesets
#' @export
#'
#' @examples
#' ## Load generic GO-slim terms
#' require(RITANdata)
#' load_geneset_symbols("GO_slim_generic")
#' print(length(active_genesets))
#' print(head(active_genesets[[1]]))
#' 
#' \dontrun{
#' ## load the default set of resources into "active_genesets"
#' load_geneset_symbols()
#' 
#' ## Use only the Reactome Pathways annotation resource.
#' load_geneset_symbols(gmt="ReactomePathways")
#' 
#' ## Suppresses output message describing the annotation resource and size.
#' load_geneset_symbols(gmt="ReactomePathways", verbose=FALSE)
#' 
#' ## To list the available resources within RITAN:
#' print(names(geneset_list))
#' 
#' ## You can also load your own data
#' load_geneset_symbols(gmt="myFile.gmt")
#' }
load_geneset_symbols <- function( gmt = NA, gmt_dir = '', verbose = TRUE ){
  sets <- NULL

  ## Cleanup inputs
  gmt_dir <- sub( '[/\\]+$', '', gmt_dir )

  if ( all(is.na(gmt)) ){
    ## ---------------------------------- -
    ## Default genesets
    sets <- do.call( c, geneset_list[ resources.default ])
    if (verbose){
      warning( sprintf('\nNo input annotation resource was selected. Defaulting to:\n%s',
                       paste(resources.default, sep=', ', collapse=', ') ))
    }

  } else if ( all(is.character(gmt)) && all( gmt %in% names(geneset_list) ) ){
    ## ---------------------------------- -
    ## Check if it's a known gene list
    sets <- do.call( c, geneset_list[ gmt ] )
    if (verbose){
      if ( all(nchar(gmt) < 100) ){
        cat(sprintf('Loading the requested genesets of "%s"...\n', gmt))
      } else {
        cat('Loading the requested genesets...\n')
      }
    }

  } else {

    ## ---------------------------------- -
    ## Loop over the requests and handle each.
    fc <- file.exists(gmt) | file.exists(sprintf('%s/%s', gmt_dir, gmt)) # file check
    nc <- gmt %in% names(geneset_list) # name check

    if (any(!( fc | nc ))){
      stop(sprintf('some resources were not recognized:\n%s', gmt[ !(fc|nc) ]))
    }

    res <- list()
    for (i in 1:length(gmt)){
      if (verbose){ cat(sprintf('Loading %s\n', gmt[i] )); }

      if (fc[i]){

        ## Load the indicated file
        this.f <- gmt[i]
        if (!file.exists(this.f)){
          this.f <- paste0( gmt_dir, '/', this.f )
        }
        res[[i]] <- readGMT(this.f)

      } else if (nc[i]){

        ## Load the known resource
        res[[i]] <- geneset_list[[ gmt[i] ]]

      }

    }

    sets <- do.call( c, res )

  }
  
  if ( (length(sets) == 0) && all(is.null(sets)) ){
    stop('No genesets loaded. Please check that your requested genesets are either:
         1) "%in% names(geneset_list)"
         2) are valid .gmt format files.'
         )
  }
  
  ## ---------------------------------- -
  ## load the gene set into global environment
  assign("active_genesets", sets, envir = .GlobalEnv)
  if (verbose){
    cat(sprintf( '\n\tLoaded %d genesets.\n', length(active_genesets) ))
  }
  
  # return() # omit explicit call to avoid the NULL print

}

#' show_active_genesets_hist
#'
#' function to plot distribution of size of active_genesets object
#'
#' @param nbins Number of bins to include in histogram
#' @param ... further argumants are passed on to plot()
#'
#' @return NULL. The plot is shown.
#' @export
#'
#' @examples
#' require(RITANdata)
#' load_geneset_symbols('GO_slim_generic')
#' show_active_genesets_hist()
#' 
#' \dontrun{
#' ## Show the distribution of geneset sizes for the default set of geneset resources
#' load_geneset_symbols()
#' show_active_genesets_hist()
#' 
#' ## Show the distribution of geneset sizes for a specific resource
#' load_geneset_symbols(gmt="ReactomePathways")
#' show_active_genesets_hist()
#' 
#' }
show_active_genesets_hist <- function(nbins = 50, ...){

  if (! exists("active_genesets")) {
    stop('The variable "active_genesets" is not set. See load_geneset_symbols().')
  }

  h <- hist( sapply(active_genesets,length), nbins, plot = FALSE )
  z <- h$density <= 0
  for (n in c('counts','density','mids')){
    h[[n]][z] <- NA # make NA so that it works with log()
  }

  plot( h$mids, h$counts, log='y', type = 'n',
        ylab = '# genesets', xlab = '# genes contained',
        main = 'Size of "active_genesets"', ... )

  for (i in 1 : (length(h$breaks)-1) ){
    rect( xleft = h$breaks[i], xright = h$breaks[i+1], ybottom = 0.1,
         ytop = h$counts[i], border = 'black', col='gray80' )
  }
  box()

}


### --------------------------------------------------------------- -
###
#' load_all_protein_coding_symbols
#' 
#' The character array returned is, by default, all human protein coding gene symbols. This variable defines the "universe of possible genes" for use in enrichment. Users should load a different "universe" or filter this one down to the most appropriate setting for their current study. For example, if running RNA-Seq, genes are in the universie if they are detected in any sample.
#' 
#' @import EnsDb.Hsapiens.v86 GenomicFeatures ensembldb AnnotationFilter
#' 
#' @return A unique list of gene symbols for protein coding genes according to EnsDb.Hsapiens.v86
#' 
load_all_protein_coding_symbols <- function(){

  ## formerly, this function read the EBI file, causing issues with timeouts.
  ## moved to using BioC package references
  # file = 'ftp://ftp.ebi.ac.uk/pub/databases/genenames/new/tsv/locus_groups/protein-coding_gene.txt',
  # col_name = 'symbol'
  # s <- read.table( file, sep="\t", header=TRUE, as.is=TRUE, quote="", comment.char = '' )
  # u <- unique( s[, col_name ])
  
  tx. <- GenomicFeatures::transcripts(EnsDb.Hsapiens.v86,
           columns = c(ensembldb::listColumns(EnsDb.Hsapiens.v86 , "tx"), "gene_name"),
           filter = AnnotationFilter::TxBiotypeFilter("protein_coding"),
           return.type = "DataFrame")
  u <- sort(unique(tx.$gene_name))
  
  return(u)
  
}

### --------------------------------------------------------------- -
### perform enrichment on gene symbols
#' enrichment_symbols
#'
#' This function is called by term_enrichment() and term_enrichment_by_subset(). The user may call it directly, but we suggest using term_enrichment(). The function uses the resources currently loaded into the active_genesets vector. See load_geneset_symbols().
#'
#' Outputs a data frame containing the gene set name, a hypergeometric-test p value, the number of genes from the
#' input gene list that occur in the gene set, the number of genes in the gene set, the gene symbols for
#' the genes in the input gene list, and the q value.
#'
#' @param geneset vector of gene symbols to be evaluated
#' @param term a list containing specific gene set term(s) and their corresponding gene symbols contained in one of the annotation resources, default is all gene set terms
#'  in the GO, ReactomePathways, KEGG_filtered_canonical_pathways, and MSigDB_Hallmarks libraries
#' @param all_symbols gene symbols to be evaluated, identified by gene symbol name. Default is all protein coding genes. This parameter should be manipulated to include only the gene symbols that pertain to the user's analysis.
#' @param ... additional arguments are not used
#'
#' @return results matrix of input gene list compared to active gene sets. Q value is calculated using entire group of active gene sets.
#' @export
#'
#' @examples
#' require(RITANdata)
#' myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
#' 
#' \dontrun{
#' ## We suggest using term_enrichment() instead. E.g.:
#' e <- term_enrichment(myGeneSet, 'GO')
#' }
#' 
#' ## But, you may use enrichment_symbols() directly for an individual term:
#' load_geneset_symbols('GO')
#' e <- enrichment_symbols(myGeneSet, 'DNA_repair', all_symbols = cached_coding_genes)
#' print(e)
#' 
#' \dontrun{
#' ## Gene set enrichment using intersection of gene symbols 
#' ##   provided in geneset parameter and all protein coding genes.
#' enrichment_symbols(geneset = vac1.day0vs31.de.genes)
#' 
#' ## choose which terms to evaluate
#' t <- active_genesets[1:5]
#' 
#' ## Test enrichment of that set of terms
#' enrichment_symbols(geneset = vac1.day0vs31.de.genes, term = t)
#' }

enrichment_symbols <- function( geneset, term = NULL, all_symbols = NA, ... ){
    
    if (! exists("active_genesets")) {
      load_geneset_symbols()
    }

    if (all(is.na(all_symbols))){
      all_symbols <- load_all_protein_coding_symbols()
    }

    geneset <- geneset[ geneset %in% all_symbols ]
    K <- length(all_symbols) # length( setdiff( kit, c('.') ) ) # assayable
    k <- length(geneset) # our observation
    
    if (is.null(term)){

      ## Test all genesets
      active_int_term <- names(active_genesets)
      p <- array( NA, dim=c( length(active_genesets), 4 ) )
      for (i in 1:length(active_genesets)){

          this.set.possible <- (active_genesets[[i]])[ active_genesets[[i]] %in% all_symbols ]

          q <- sum( geneset %in% this.set.possible )
          p[i,2] <- q
          p[i,3] <- paste( geneset[ geneset %in% this.set.possible ], collapse=';' )
          p[i,4] <- length(unique( this.set.possible ))
          if ( q >= 2 ){
              m <- as.numeric(p[i,4])
              n <- K - m
              p[i,1] <- phyper( q-1, m, n, k, lower.tail = FALSE )
          } else {
              p[i,1] <- 1
          }

      }

    } else {
      
      if (is.null(names(term))){
        active_int_term <- intersect(names(active_genesets), term)
        if ( length(active_int_term) == 0 ){
          active_int_term <- names(active_genesets)[ grepl( sprintf('[[:alnum:]]*[.]%s$',term), 
                                                            names(active_genesets) ) |
                                                     grepl( sprintf('[[:alnum:]]*[.]%s$', gsub(' ','_',term) ), 
                                                            names(active_genesets) )
                                                    ]
        }
      } else {
        active_int_term <- intersect(names(active_genesets), names(term))
      }
      
      if ( length(active_int_term) < max(c( length(term), length(names(term)) )) ) {
        cat(sprintf('Undefined Term: %s', setdiff(term, active_int_term) ), sep = "\n")
        stop('You have requested terms that have not been defined. Please check contents of "term" parameter.')
      } else {
        
        ## Test the requested term
        p <- array( NA, dim=c( length(active_int_term), 4 ) )
        for (i in 1:length(active_int_term)){
          
          j <- which(names(active_genesets) == active_int_term[i])
          this.set.possible <- (active_genesets[[j]])[ active_genesets[[j]] %in% all_symbols ]
          q <- sum( geneset %in% this.set.possible )
          p[i,2] <- q
          p[i,3] <- paste( geneset[ geneset %in% this.set.possible ], collapse=';' )
          p[i,4] <- length(unique( this.set.possible ))
          if ( q >= 2 ){
            m <- as.numeric(p[i,4])
            n <- K - m
            p[i,1] <- phyper( q-1, m, n, k, lower.tail = FALSE )
          } else {
            p[i,1] <- 1 # if only one gene, set the p-value to 1 - not quite right, but singletons were leading to numeric errors...
          }
          
        }
      }
        
    }
    
    # tmp <- names(active_genesets)
    # if (!is.null(term)){
    #     tmp <- names(term)
    # }

    res <- data.frame( name  = active_int_term,
                       p     = as.numeric(p[,1]),
                       n     = as.numeric(p[,2]),
                       n.set = as.numeric(p[,4]),
                       genes = p[,3],
                       q     = p.adjust( as.numeric(p[,1]), method = "BH" )  )
    
    #if (dim(res)[1] < 100){
    #  warning('Few terms were considered here.
    #The q-values returned may not be appropriate for this particular analysis')
    #}

    res <- res[order(res$p), ]

    return(res)
}


term.test <- function( set, sub_set, total ){
    K <- length(total) # space of possibilities
    k <- length(set) # our observation size
    q <- sum( set %in% sub_set ) # observed hits
    m <- length(sub_set) # possible hits
    n <- K - m
    return( phyper( q-1, m, n, k, lower.tail = FALSE ) )
}

### --------------------------------------------------------------- -
#' geneset_overlap
#' 
#' Return assymetric matrix of the fraction of genes shared between sets. E.G. The fraction of the first set that is "covered" by or "overlaps" the second set.
#'
#' @param s1 The first geneset
#' @param s2 the second geneset
#' @param s.size Denominator used in each comparison. The default is to determint the lengths of elements in "s1"
#'
#' @return results matrix of input gene list compared to active gene sets. Q value is calculated using entire group of active gene sets.
#' @export
#'
#' @examples
#' require(RITANdata)
#' r <- geneset_overlap( geneset_list$MSigDB_Hallmarks, geneset_list$NetPath_Gene_regulation )
#' heatmap(r, col = rev(gray(seq(0,1,length.out = 15))) )
#' summary(c(r))
geneset_overlap <- function( s1, s2 = s1, #threshold = 0.5,
                             s.size = unlist(lapply( s1, length )) ){
    
    n <- length(s1)
    m <- length(s2)
    r <- array( 0, dim=c(n,m) )
    for (i in 1:n){

        for (j in 1:m ){
            r[i,j] <- sum( s1[[i]] %in% s2[[j]] ) / s.size[i] #length( s[[i]] )
        }

    }
    
    if (!all(is.null(names(s1)))){
      rownames(r) <- names(s1)
    }
    
    if (!all(is.null(names(s2)))){
      colnames(r) <- names(s2)
    }
    
    return(r)

}


#' resource_reduce
#' Merge terms across resources to reduce the number of redundant and semi-redundant terms
#'
#' @param genesets the input genesets to consider. May be from one or multiple resources.
#' @param min_overlap terms that share at least this fraction of genes will be merged
#' @param verbose if TRUE, print status and summary output 
#' 
#' @return the list of terms, after merging to reduce redundnat and semi-redundant terms
#' @export
#' 
#' @examples
#' require(RITANdata)
#' r <- resource_reduce( geneset_list$DisGeNet )
#'
resource_reduce <- function( genesets = NULL, min_overlap = 0.8, verbose = TRUE ){
  
  if (all(is.null(genesets))){
    stop('no terms/genesets were provided')
  }
  
  if ( (length(genesets) > 2000) & verbose ){
    warning(sprintf( 'This function is considering all pairwise relationships among the %d input terms/genesets.
This may take some time...', length(genesets) ))
  }
  
  o <- geneset_overlap(genesets)
  diag(o) <- NA # ignore self-overlap
  i <- which( o > min_overlap, arr.ind = TRUE )
  
  all_terms <- unique(c(rownames(o), colnames(o)))
  terms_with_overlap <- unique(c( rownames(o)[ i[,1] ], colnames(o)[ i[,2] ] ))
  
  merge_list <- new_genesets <- list()
  seen <- rep(FALSE, length(all_terms))
  gi <- 0
  for (r in 1:dim(i)[1]){
    
    ## initilize new group
    if (seen[ i[r,1] ] == FALSE){
      gi <- gi + 1
      seen[i[r,1]] <- seen[i[r,2]] <- TRUE
      merge_list[[gi]] <- c( rownames(o)[ i[r,1] ], colnames(o)[ i[r,2] ] )
    } else {
      next
    }
    
    ## populate group with all terms/sets that are highly related
    j <- (!seen) & ( ( all_terms %in% colnames(o)[ i[ i[,1] == i[r,1] , 2] ] ) |
                     ( all_terms %in% rownames(o)[ i[ i[,2] == i[r,2] , 1] ] ) )
    while( any(j) ){
      
      merge_list[[gi]] <- sort(unique(c( merge_list[[gi]], all_terms[j] )))
      seen[j] <- TRUE
      j <- (!seen) & ( ( all_terms %in% colnames(o)[ i[ i[,1] == i[r,1] , 2] ] ) |
                       ( all_terms %in% rownames(o)[ i[ i[,2] == i[r,2] , 1] ] ) )
      
    }
    
    ## generate the merged geneset
    new_genesets[[gi]] <- sort(unique(c(unlist( genesets[ merge_list[[gi]] ] ))))
    names(new_genesets)[gi] <- paste( merge_list[[gi]], sep=';', collapse = ';' )
    
  }
  
  out <- c( new_genesets, genesets[ !(names(genesets) %in% terms_with_overlap) ] )
  
  if (verbose){
    cat(sprintf('
The input list had %d terms/genesets.
The %d terms with overlaps of %.2f were merged into %d composite terms.
The updated term list with %d terms will be returned.
',
                length(genesets), length(terms_with_overlap), min_overlap, length(new_genesets), length(out) ))
  }
  
  return(out)
  
}



### --------------------------------------------------------------- -
### Enrichment analysis
#' term_enrichment
#'
#' term_enrichment evaluates the input gene list for enrichment within each of the annotation resources. This differs from the enrichment_symbols function which evaluates the gene list for enrichment against all of the annotation resources grouped together.
#'
#' @param geneset vector of gene symbols to be evaluated
#' @param resources list containing the reference gene sets to test for enrichment
#' @param report_resources_separately logical (default FALSE) flag to report enrichments seperately for each requested resource, or to combine them and produce FDR adjustment across the combined set
#' @param verbose print the top results for each annotation resource
#' @param all_symbols the background/global set of gene symbols (study dependent; we provide all protien coding genes as a default)
#' @param filter_to_intersection [FALSE] should the background and foreground genesets be subsetted to one another?
#' @param ... further arguments are passed on to enrichment_symbols()
#' 
#' @return results matrix of input gene list compared to active gene sets. Q value is calculated within each of the active gene sets.
#' @import RITANdata
#' @export
#'
#' @examples
#' ## Check if there is enrichment for any "Hallmark" functions within a input set of genes
#' require(RITANdata)
#' myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
#' e <- term_enrichment(myGeneSet, "MSigDB_Hallmarks")
#' print( e[1:2, -6] )
#' 
#' \dontrun{
#' term_enrichment(geneset = vac1.day0vs31.de.genes)
#' term_enrichment(geneset = vac1.day0vs31.de.genes, resources = "MSigDB_Hallmarks")
#' vac1.day0v31.enrichment <- term_enrichment(geneset = vac1.day0vs31.de.genes, verbose = FALSE)
#' }
term_enrichment <- function( geneset, resources = resources.default,
                             report_resources_separately = FALSE,
                             verbose = TRUE, all_symbols = NA,
                             filter_to_intersection = FALSE, ... ){
  
  if ( (!exists('all_symbols')) || all(is.na(all_symbols)) ){
    all_symbols <- load_all_protein_coding_symbols()
  }
  
  enrich <- list()
  process_source <- function( s_file, v=TRUE, f_intersect=FALSE ){
    
    load_geneset_symbols( s_file, verbose = v ) # load into active_genesets
    
    if (f_intersect){
      all_symbols_active <- unique(c(unlist( active_genesets ))) # baseline background
      ## -->> Need to detect if user gave "all_symbols" and take intersection <<--
      ## -->> Also, limit "geneset" to intersection <<--
      if (!is.null(get('all_symbols'))){
        all_symbols_active <- all_symbols_active[ all_symbols_active %in% all_symbols ]
      }
      geneset <- geneset[ geneset %in% all_symbols_active ]
    } else {
      all_symbols_active <- all_symbols
    }

    # run enrichment for active_genesets
    p0 <- enrichment_symbols( geneset, term = names(active_genesets),
                              all_symbols = all_symbols_active, ... )
    p0 <- p0[ order(p0$p), ]

    if (v){
      #printBanner( s_file )
      print( head( p0[ ,-5], n=5 ) )
    }

    return(p0)

  }

  if( report_resources_separately ){

    enrich <- lapply( resources, function(x){
                      process_source(x, v = verbose,
                                     f_intersect = filter_to_intersection) })
    names(enrich) <- resources

  } else {

    enrich <- process_source( resources, v = verbose,
                              f_intersect = filter_to_intersection )

  }
  
  class(enrich) <- c( 'term_enrichment', class(enrich) )
  attr(enrich, 'resources') <- resources
  
  return(enrich)

}

### --------------------------------------------------------------- -
### Enrichment analysis
#' term_enrichment_by_subset
#'
#' Run enrichment simultaneously across a group of prioritized gene lists. For example, in a time course dataset, one may have a different list of genes that are differentially expressed at each time point. This function facilitates rapid evaluation of term enrichment across time point comparisons. Alternatively, one may have a different list of differentially expressed genes by drug treatment, environmental condition, ect.
#'
#' @param groups A list() of genes for enrichment. Each entry in the list() is an input set of genes. Enrichment is performed for each of these entries.
#' @param resources character vector for which resources to use in enrichment
#' @param q_value_threshold minimum q-value (FDR adjusted p-value) in any group for the term to be included in results
#' @param verbose print additional status updates on what the function is doing
#' @param display_type Flag for which data type will be returned. One of "q" (default) for q-values, "p" for unadjusted p-values, or "n" for the number of genes overlapping the term.
#' @param phred Logical flag (default TRUE) to return the -log10 of p/q values
#' @param ... Further arguments are passed on to enrichment_symbols()
#'
#' @return Returns a term-by-study matrix of enrichment values (value determined by "display_type")
#' @import RITANdata
#' @importFrom methods is
#' @export
#'
#' @examples
#' ## Create list of gene sets to evaluate.
#' ##   This example is from a vaccine study where we pre-generated differentially expressed genes.
#' ##   This object will be passed to the groups parameter.
#' require(RITANdata)
#' vac1.de.genes <- list(vac1.day0vs31.de.genes, vac1.day0vs56.de.genes)
#' names(vac1.de.genes) <- c("Day0vs31", "Day0vs56")
#' print(str(vac1.de.genes))
#' 
#' \dontrun{
#' ## Run term_enrichment_by_subset on the two results.
#' ##   This function usually takes a few seconds to a minute to run.
#' m <- term_enrichment_by_subset(groups = vac1.de.genes, q_value_threshold = .9)
#' summary(m)
#' plot( m, label_size_y = 4, show_values = FALSE )
#' }
#' 
term_enrichment_by_subset <- function( groups = NA, resources = resources.default,
                                       q_value_threshold = 0.01, verbose = TRUE,
                                       display_type = 'q', phred=TRUE, ... ){

  ## Make a matrix for the term enrichment of each "group" (a list where each entry is a vector of genes)
  ## "Extra" arguments are passed to enrichment_symbols()

  if ( all(is.na(groups)) || (!methods::is(groups, 'list')) ){
      stop( '"groups" should be a list where each entry is a vector of gene symbols.' )
  }
  if ((display_type == 'n') && ( phred == TRUE ) ){
    warning('"display_type" is set to "n" and "phred" is TRUE. We suggest setting phred=FALSE.\n')
  }
  
  ## Load the requested resources
  ## This will load them as a group into "active_genesets"
  load_geneset_symbols(resources, verbose = verbose)
  
  ## get enrichments (each is sorted by p)
  ## This function acts on "active_genesets"
  if(verbose){ cat("\tRunning Enrichment...", sep=" ") }
  e <- lapply( groups, enrichment_symbols, ... ) # optional arguments are sent on

  ## sort them together
  term.order <- e[[1]]$name
  e <- lapply( e, function(x){ x[ match(term.order,x$name), ] })

  ## filter by q-value
  if(verbose){ cat("FDR adjustment...", sep=" ") }
  i <- apply( do.call( cbind, lapply( e, function(x){ getElement(x,'q') <= q_value_threshold } ) ), 1, any )
  m <- cbind( e[[1]][ , c('name', 'n.set' )],
              do.call( cbind, lapply( e, function(x){ getElement(x, display_type) } ) )
              )
  first.data.col <- 3
  data.cols <- first.data.col : dim(m)[2]
  m <- m[ i, ]
  m <- m[ order(apply(m[, data.cols ], 1, min)), ]

  if( phred ){
      m[ , data.cols ] <- -log10(m[ , data.cols ])
  #} else {
  #    m <- m[ order( m[,first.data.col], decreasing=FALSE ), ]
  }

  if ( phred || (display_type == 'n')){
    m <- m[ order( apply(m[,data.cols], 1, max), decreasing=TRUE ), ]
  }

  m[ , data.cols ] <- round( m[ , data.cols ], 1 )
  class(m) <- c( 'term_enrichment_by_subset', class(m) )
  attr(m, 'display_type') <- display_type
  attr(m, 'phred') <- phred
  
  if(verbose){ cat("done.", sep="\n") }
  return( m )

}


#' plot.term_enrichment
#' @param x data frame returned by term_enrichment
#' @param min_q Only q-values more significant than this threshold will be plotted. Default = 0.05.
#' @param max_terms Up to max_terms will be plotted. Default = 25.
#' @param extend_mar Term names can be long. We attempt to keep them readable by extending the left-hand-side margins automatically. Default = c(0,10,0,0) added to par()$mar.
#' @param ... Additional arguments are passed on to plot()
#' @return silent return from plot
#' @import RITANdata graphics utils gplots png
#' @rawNamespace import(plotrix, except=c('plotCI'))
#' @rawNamespace import(stats, except=c('decompose','lowess','spectrum'))
#' @export
#' 
#' @examples
#' require(RITANdata)
#' e <- term_enrichment(vac1.day0vs31.de.genes, resources = 'GO_slim_generic')
#' plot(e, min_q = .1)
#'
plot.term_enrichment <- function( x = NA, min_q = 0.05, max_terms = 25, extend_mar = c(0,10,0,0), ... ){
  
  if ( is.null(dim(x)) || all(is.na(x)) ){
    stop('input data for plotting not found')
  }
  
  if (max_terms > dim(x)[1]){
    max_terms <- dim(x)[1]
  }
  
  i <- (x$q < min_q)
  if (!any(i)){
    stop('No terms meet the requested min_q threshold')
  }
  
  if ( length(extend_mar==4) & (par()$mar[2] < extend_mar[2]) ){
    ## The term labels can be long strings.
    ## Extent the left-hand-side margins.
    par( mar = par()$mar + extend_mar )
  }
  
  if (sum(i) > max_terms){
    i <- rep(FALSE, dim(x)[1])
    i[ which(x$q < min_q)[ 1:max_terms] ] <- TRUE
  }
  
  plot( -log10(x$q[i]), which(i),
        xlab = expression(paste('-log'[10], '( q-value )')),
        ylab = '', yaxt='n', ... )
  
  axis( side = 2, at = which(i), labels = rep('', sum(i)) )
  
  text( par()$usr[1] - 0.05*( par()$usr[2] - par()$usr[1] ), # x placement is 5% to the left of the x-axis starting
        which(i), labels = sub( '.', '\n', x$name[i], fixed=TRUE ),
        cex = 0.6, srt = 0, xpd=TRUE, adj=c(1.0,0.5) )
  
}


#' plot.term_enrichment_by_subset
#'
#' @param x data frame returned by term_enrichment_by_subset
#' @param show_values True or False, plot values on the heatmap
#' @param annotation_matrix a matrix() of group-levle characteristics - same number of columns as "m"
#' @param annotation_palates Color palates (RColorBrewer) used for each row of the annotation matrix
#' @param annotation_legend_x offset for placing the legend
#' @param low color for low end of range
#' @param high color for high end of range
#' @param label_size_x size of text for x label. Default lable_size_x=16
#' @param label_angle_x angle for text for x label. Default is -30 degrees
#' @param label_size_y size of text for y label. Default label_size_y=9
#' @param grid_line_color color o grid lines between cells. Default is white.
#' @param mid sets lower threshold for color scale
#' @param wrap_y_labels Number of characters to wrap row labels
#' @param cap Clip numeric values to this maximum threshold
#' @param return_ggplot_object logical flag (default FALSE) that if TRUE, the ggplot object for the plot is returned
#' @param trim_resource_names [TRUE] remove any text in rownames preceeding a period characte. This convension is usually used in RITAN to prepend the resource name to the term name, which may not be needed in plotting.
#' @param ... further areguments are not used at this time. If the user wants to modify the plot, use return_ggplot_object = TRUE.
#'
#' @return silent return, unless return_ggplot_object==TRUE. Then, the ggplot object for the plot is returned.
#' @export
#' @import ggplot2 reshape2 grid gridExtra RColorBrewer
#' @importFrom methods is
#' 
#' @examples
#' ## Create list of gene sets to evaluate.
#' ##   This example is from a vaccine study where we pre-generated differentially expressed genes.
#' ##   This object will be passed to the groups parameter.
#' require(RITANdata)
#' vac1.de.genes <- list(vac1.day0vs31.de.genes, vac1.day0vs56.de.genes)
#' names(vac1.de.genes) <- c("Day0vs31", "Day0vs56")
#' print(str(vac1.de.genes))
#' 
#' \dontrun{
#' ## Run term_enrichment_by_subset on the two results.
#' ##   This function usually takes a few seconds to a minute to run.
#' m <- term_enrichment_by_subset(groups = vac1.de.genes, q_value_threshold = .9)
#' summary(m)
#' plot( m, label_size_y = 4, show_values = FALSE )
#' }
#'
plot.term_enrichment_by_subset <- function( x, show_values = TRUE,
                                            annotation_matrix = NA,
                                            low = 'white', high="#2166AC",
                                            return_ggplot_object = FALSE,
                                            label_size_x = 16, label_angle_x=-30,
                                            label_size_y = 9, wrap_y_labels = 20,
                                            grid_line_color = 'white', mid = 0, cap = NA,
                                            annotation_palates = c('Reds','Greens','Purples','Greys','BuPu','RdPu','BrBG','PiYG','Spectral'),
                                            annotation_legend_x = -0.3, 
                                            trim_resource_names = TRUE, ... ){

  ## Check inputs
  if ( all(is.na(x)|is.null(x)) || is.null(dim(x)) || (dim(x)[1] < 1) ){
    stop(' insufficient data provided to plot.term_enrichment_by_subset()')
  }
  display_type <- attr(x, 'display_type')
  phred <- attr(x, 'phred')

  if ( (!all(is.na(annotation_matrix))) &&
       ((dim(x)[2] - dim(annotation_matrix)[2]) != 2 ) ){
    stop(sprintf('You have requested to add an annotation matrix, but the dimentions do not line up.\nExpected: %d',
                 dim(x)[2]-2, dim(annotation_matrix)[2] ))
  }

  ## Optionally, cap the values for plotting
  if ( (!all(is.na(cap))) & methods::is(cap, 'numeric') ){
    if (length(cap) > 1){ stop('"cap" should be a single number') }
    x[, 3:dim(x)[2] ] <- apply( x[, 3:dim(x)[2] ], c(1,2), function(y){
      if (y > cap){ return(cap)
      } else {      return(y)
      }
    })
  }

  ## make matrix with row/column names
  mat <- as.matrix( x[ , 3:dim(x)[2] ] )
  colnames(mat) <- colnames(x)[ 3:dim(x)[2] ]

  ## trim off the "resource" from term names for plotting
  if (trim_resource_names){
    rownames(mat) <- as.character(sapply(
      getElement( x, 'name' ), function(x){
        sub( '.+[.]', '', x )
      }))
  }

  ## basic 1-level name wrapping for plotting
  if ( !all(is.na(wrap_y_labels)) && is.numeric(wrap_y_labels) ){
    rownames(mat) <- as.character( sapply( rownames(mat), function(x){
        if( nchar(x) > wrap_y_labels ){
          y <- strsplit( x, '' )[[1]]
          i <- grep( '[-_ ]', y )
          j <- wrap_y_labels
          if(any(i)){
            j <- i[ which.min( abs(i-wrap_y_labels) ) ]
          }
          x <- paste( c(y[1:j], '\n', y[ (j+1) : length(y) ]), sep='', collapse = '' )
        }
        return(x)
      }))
  }


  ## make ggplot2 compatible
  #dat <- reshape2::melt( mat )
  #levels(dat$Var1) <- rownames(mat)
  #levels(dat$Var2) <- colnames(mat)
  
  ## make ggplot2 compatible - updated
  dat <- reshape2::melt( mat, varnames=c('Var1', 'Var2') )
  levels(dat$Var1) <- rownames(mat)
  levels(dat$Var2) <- colnames(mat)
  
  ## It would be nice to add a >= sign to the legend...
  #if ( show_values & (!all(is.na(cap))) & is(cap, 'numeric') ){
  #  dat$value[ dat$value == cap ] <- sprintf( '\u2265%f', dat$value )
  #}

  ## make ggplot heatmap
  g <- ggplot( dat, aes(Var2, Var1)) +
    geom_tile( aes_string(fill = 'value' ), colour=grid_line_color )
  if (show_values){
     g <- g + geom_text( label = round(dat$value, 1) ) # aes_string(fill = 'value' )
  }
  g <- g + scale_fill_gradient2(midpoint=mid, low=low, high=high)
  g <- g + theme( axis.ticks = element_blank(),
                  axis.text.x = element_text(size = label_size_x, angle = label_angle_x,
                                             colour = "black", hjust = 0),
                  axis.text.y = element_text(size = label_size_y,
                                             colour = "black"),
                  panel.background = element_blank()
  ) + xlab('') + ylab('')


  ## Check if the user has supplied a sample/study/column annotation matrix
  if (!all(is.na(annotation_matrix))){

    mc <- max(apply(annotation_matrix, 1, function(x){ length(unique(x)) }))
    if (mc > 8){
      stop('Generation of the annotation_matrix visualization currently uses RColorBrewer.
These color palates are only defined for up to 9 colors (including white).
Please modify your annotation_matrix to have up to 8 values per row.')
    }

    ## set color palates to use. Repeat Spectral as necessary
    nr <- dim(annotation_matrix)[1]
    if ( nr > length(annotation_palates)){
      d <- nr - length(annotation_palates)
      annotation_palates <- c(annotation_palates, rep('Spectral', d) )
    }

    ## Translate data into color values
    dat <- reshape2::melt(annotation_matrix); names(dat) <- c('Var1','Var2','value')
    levels(dat$Var1) <- rownames(annotation_matrix)
    levels(dat$Var2) <- colnames(annotation_matrix)
    dat$col <- dat$colID <- NA
    uc <- ul <- NULL
    ci <- 0
    for (i in 1:nr){

      nc <- length(unique(annotation_matrix[i,]))
      row_col <- brewer.pal( n = nc+1, name = annotation_palates[i] )[-1]
      uc <- c(uc, row_col)
      ul <- c(ul, unique(annotation_matrix[i,]) )
      for (j in 1:nc){
        ci <- ci+1

        k <- (dat$Var1 == rownames(annotation_matrix)[i]) &
             (dat$value == unique(annotation_matrix[i,])[j] )
        dat$col[k] <- row_col[j]
        dat$colID[k] <- ci
      }

    }
    dat$colID <- factor(dat$colID)

    ## Make ggplot
    ggmat <- ggplot( dat, aes(Var2,Var1)) +
      geom_tile( aes_string(fill = 'colID' ), colour=grid_line_color ) +
      scale_fill_manual( values = uc, labels = ul ) +
      theme( axis.ticks = element_blank(),
             axis.text.y = element_blank(),
             axis.text.x = element_text(size = 8, angle = -45, hjust = 0),
             panel.background = element_blank(),
             legend.spacing = unit(0, "cm"), legend.text = element_text(size=rel(0.5)),
             title = element_blank(), legend.key.size = unit(.3,'cm'),
             legend.position = c(annotation_legend_x, 0.5)
      ) + xlab('') + ylab('')

    ## Combine with heatmap
    p1 <- ggplotGrob(g)
    p2 <- ggplotGrob(ggmat)
    p2$widths <- p1$widths
    g <- gridExtra::arrangeGrob( p2, p1, heights = c(1,4), ncol=1 )
    
    ## Return or Plot
    if (return_ggplot_object){
      #g <- arrangeGrob( p2, p1, layout_matrix = matrix(c(1, rep(2,4) ), ncol=1) )
      return(g)
    } else {
      plot(g)
    }

  } else {

    ## Return or Plot
    if (return_ggplot_object){
      return(g)
    } else {
      plot(g)
    }

  }

}


#' summary.term_enrichment
#'
#' @param object data frame returned by term_enrichment()
#' @param ... Further arguments are passed on to head()
#' 
#' @return the data.frame of top enrichment results
#' @export
#' 
#' @examples
#' require(RITANdata)
#' e <- term_enrichment( vac1.day0vs31.de.genes, "MSigDB_Hallmarks" )
#' summary(e, n=3)
#' 
summary.term_enrichment <- function(object, ...){
  
  if ( 'list' %in% class(object) ){
    lapply( object, function(y){
      print(head(y[,-5], ...))
    })
  } else {
    print(head(object[,-5], ...))
  }
  
}


#' summary.term_enrichment_by_subset
#'
#' @param object data frame returned by term_enrichment_by_subset()
#' @param verbose if TRUE (default), print a header describing the data type
#' @param ... Further arguments are passed on to head()
#' 
#' @return the data.frame of top enrichment results
#' @export
#' 
#' @examples
#' require(RITANdata)
#' vac1.de.genes <- list(vac1.day0vs31.de.genes, vac1.day0vs56.de.genes)
#' names(vac1.de.genes) <- c("Day0vs31", "Day0vs56")
#' e <- term_enrichment_by_subset(vac1.de.genes, "MSigDB_Hallmarks", q_value_threshold = 0.1 )
#' summary(e)
#'
summary.term_enrichment_by_subset <- function(object, verbose = TRUE, ...){
  
  if (verbose){
    # attributes(object)
    
    dt <- attr(object, 'display_type')
    ph <- attr(object, 'phred')
    
    if (dt == 'p'){
      
      ph_tag <- ifelse( ph, '-log10(p)', 'p' )
      print(sprintf('The data matrix contains %s-values which have not been adjusted for multiple testing', ph_tag))
      
    } else if (dt == 'q'){
      
      ph_tag <- ifelse( ph, '-log10(q)', 'q' )
      print(sprintf('The data matrix contains %s-values which have been adjusted for multiple testing (FDR)', ph_tag))
      
    } else if (dt == 'n'){
      
      print('The data matrix contains the number of genes in the query that are contained within each subset')
      
    } else {
      print( sprintf('Data type "%s" not recognized.', as.character(dt)) )
    }
    
    print('Each term has its own row and each subset/substudy is a column')
    
  }
  
  head(object, ...)
  
}

## ----first_load, echo=TRUE, warning=FALSE, results='hide', message=FALSE------
library(RITANdata)
library(RITAN)
library(knitr)
kable( attr(network_list, 'network_data_sources') )
kable( sapply(geneset_list, length), col.names = c('# Genesets') )

## ----geneset_overlap_P, echo=TRUE, eval=FALSE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# genes <- geneset_list$MSigDB_C5$APICAL_JUNCTION_COMPLEX
# e <- network_overlap( genes, resources = c('CCSB','STRING'), minStringScore = 700 )
# ## We also strongly encourage use of the BioPlex database, which we do not distribute with RITAN in compliance with their licensing.

## ----geneset_overlap_M, echo=TRUE, eval=FALSE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# genes2 <- geneset_list$MSigDB_C5$AMINE_METABOLIC_PROCESS
# e2 <- network_overlap( genes, resources = c('PID','HumanNet') )

## ----geneset_overlap_E, echo=TRUE, eval=FALSE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# genes2 <- geneset_list$MSigDB_C5$AMINE_METABOLIC_PROCESS
# e2 <- network_overlap( genes, resources = c('ChEA','HumanNet') )

## ----first_load, echo=TRUE, warning=FALSE, results='hide', message=FALSE------
library(RITANdata)
library(RITAN)

## ----geneset_overlap_d, echo=TRUE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# Show the fraction of genes common to multiple disease-gene relationships.
o <- geneset_overlap( geneset_list$DisGeNet )
plot( density(c( o[upper.tri(o)] )), log='y', ylim = c(1e-3, 1e3),
      main='', xlab='Fraction Shared Genes', ylab='Prob()')

## ----geneset_overlap_h, echo=TRUE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# Show the diseases and their gene-level overlap, when each disease has some overlap of at least 80%.
diag(o) <- NA # ignore self-overlap
i <- which( o > 0.8, arr.ind = TRUE )
mat <- o[ unique(i[,1]), unique(i[,2]) ]
heatmap( mat, col = rev(gray(seq(0,1,length.out = 15))),
         cexRow=.7, cexCol=0.7, margins = c(7,7) )

## ----geneset_overlap2_d, echo=TRUE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# Show the fraction of genes common between disease-gene relationships and GO-Slim term definitions
o <- geneset_overlap( geneset_list$DisGeNet, geneset_list$GO_slim_generic )
o <- o[ , !( colnames(o) %in% c('biological_process', 'molecular_function', 'cellular_component')) ] # remove the root of each sub-ontology
plot( density(c(o)), log='y',
      main='', xlab='Fraction Shared Genes', ylab='Prob()')

## ----geneset_overlap2_h, echo=TRUE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# Show the heatmap for relationships where a disease and term share 95% of genes in common
i <- which( o > 0.95, arr.ind = TRUE )
mat <- o[ unique(i[,1]), unique(i[,2]) ]
heatmap( mat, col = rev(gray(seq(0,1,length.out = 15))),
         cexRow=.7, cexCol=0.7, margins = c(7,7) )

## ----geneset_overlap3, echo=TRUE, warning=FALSE-------------------------------
rownames(o)[ o[ , "chromosome_organization" ] > 0.66 ]

## ----geneset_overlap4, echo=TRUE, warning=FALSE-------------------------------
d <- rownames(o)[ o[ , "cell_motility" ] > 0.66 ]
str(d)
new_geneset <- intersect( unique(unlist(geneset_list$DisGeNet[d])),
                          unique(unlist(geneset_list$GO_slim_generic$cell_motility)) )
str(new_geneset)

## ----resource_reduce_1, echo=TRUE, warning=FALSE------------------------------
unique_diseases <- resource_reduce( geneset_list$DisGeNet )

## ----resource_reduce_2, echo=TRUE, warning=FALSE------------------------------
unique_disease_slim <- resource_reduce( c(geneset_list$DisGeNet, geneset_list$GO_slim_generic), min_overlap = 0.95 )

## ----bgee1, echo=TRUE, eval=FALSE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'----
# # Load RITAN
# library(RITANdata)
# library(RITAN)
# 
# # Install the Bgee package. GO.db is a dependency of a dependency and may need to be installed seperately.
# for (pkg in c('GO.db','BgeeDB','biomaRt')){
#   if (! (pkg %in% rownames(installed.packages()) )){
#     if (!requireNamespace("BiocManager", quietly=TRUE))
#         install.packages("BiocManager")
#     BiocManager::install(pkg)
#   }
#   library(pkg, character.only = TRUE)
# }
# 
# for (pkg in c('tidyselect','venn','magrittr','ggplot2','igraph')){
#   if (! (pkg %in% rownames(installed.packages()) )){
#     install.packages(pkg)
#   }
#   library(pkg, character.only = TRUE)
# }
# 
# # Setup Bgee query & get data (this may take some time)
# bgee <- Bgee$new(species = "Homo_sapiens", dataType = "rna_seq", release = "13.2")
# data <- getData(bgee)
# e    <- formatData(bgee, data[[1]], callType = "present", stats = "rpkm")
# 
# # Explore the dataset with: str(sampleNames(e)), str(featureNames(e)), str(phenoData(e))
# table(phenoData(e)@data$Anatomical.entity.name)
# 
# ## -------------------- -
# ## Get expression in two tissues
# tmp <- exprs(e)[ , phenoData(e)@data$Anatomical.entity.name == "heart" ]
# i   <- apply( tmp, 1, function(x){ any(is.na(x)) })
# expr_heart <- tmp[ !i, ]
# 
# tmp <- exprs(e)[ , phenoData(e)@data$Anatomical.entity.name == "skeletal muscle tissue" ]
# i   <- apply( tmp, 1, function(x){ any(is.na(x)) })
# expr_skele <- tmp[ !i, ]
# 
# venn::venn( list(Heart = rownames(expr_heart),
#                  Skeletal = rownames(expr_skele) ),
#             cexil= 1, cexsn = 1, zcolor = "style" )
# 
# ## -------------------- -
# ensembl <- useMart("ensembl", dataset = "hsapiens_gene_ensembl", "http://Aug2017.archive.ensembl.org" ) # version 90
# 
# map_heart <- getBM( attributes = c('ensembl_gene_id','ensembl_transcript_id','hgnc_symbol'),
#                     filters = 'ensembl_gene_id', values = rownames(expr_heart), mart = ensembl )
# 
# map_skele <- getBM( attributes = c('ensembl_gene_id','ensembl_transcript_id','hgnc_symbol'),
#                     filters = 'ensembl_gene_id', values = rownames(expr_skele), mart = ensembl )
# 
# ## -------------------- -
# ## Functions associated with each tissue's top genes
# ## Important: the p-values reported here are observational, not inferential.
# 
# mh <- apply( expr_heart, 1, mean )
# top_heart <- map_heart$hgnc_symbol[ map_heart$ensembl_gene_id %in% rownames( expr_heart )[ mh > quantile(mh, .975) ] ] %>% setdiff(.,'')
# 
# ms <- apply( expr_skele, 1, mean )
# top_skele <- map_skele$hgnc_symbol[ map_skele$ensembl_gene_id %in% rownames( expr_skele )[ ms > quantile(ms, .975) ] ] %>% setdiff(.,'')
# 
# e <- term_enrichment_by_subset( list( Heart    = top_heart,
#                                       Skeletal = top_skele ),
#                                 resources = 'GO_slim_PIR', all_symbols = cached_coding_genes )
# 
# plot( e[ apply(e[, c(3:4)], 1, max) >= 12, ], cap=40, label_size_y = 8, wrap_y_labels = FALSE )
# 
# ## -------------------- -
# ## Network Interactions Within Each Tissue
# 
# net_h <- network_overlap( top_heart, resources = c('CCSB','dPPI','HumanNet') )
# net_s <- network_overlap( top_skele, resources = c('CCSB','dPPI','HumanNet') )
# 
# net2g <- function(x){
#   edges <- as.matrix( x[, c(1,3)] )
#   G <- igraph::make_undirected_graph( c(t(edges)) )
#   return(G)
# }
# 
# g_h <- net2g( net_h )
# g_s <- net2g( net_s )
# 
# g_dif <- igraph::difference( g_h, g_s )
# g_int <- igraph::intersection( g_h, g_s )
# 
# cat(sprintf('
# Of the top expressed genes, %d are shared and %d differ.
# ', length(V(g_int)), length(V(g_dif)) ))
# 
# par(mar=rep(0,4))
# plot(g_dif, vertex.size = 2, vertex.label = NA, vertex.frame.color = 'white', layout = layout_nicely )

## ----load_data from package, echo=TRUE, results='hide', message=FALSE---------
library(RITANdata)
library(RITAN)
require(knitr)

## ----citation, echo=TRUE------------------------------------------------------
kable( attr(network_list, 'network_data_sources') )

## ----example1, echo=TRUE------------------------------------------------------
my_genes <- geneset_list$MSigDB_C7[['GSE6269_HEALTHY_VS_FLU_INF_PBMC_DN']]
net <- network_overlap( my_genes, resources = 'CCSB' )

## ----example1.1, echo=TRUE----------------------------------------------------
head(unique(net))

## ----example2, echo=TRUE, eval=FALSE------------------------------------------
# net2 <- network_overlap( my_genes, resources = c('CCSB','STRING'), minStringScore = 700 )
# str(net2)

## ----check_input1, echo = TRUE------------------------------------------------
my_genes <- geneset_list$MSigDB_C2[['VERNOCHET_ADIPOGENESIS']]
i <- check_any_net_input( my_genes )
table(i)

## ----check_input2, echo = TRUE------------------------------------------------
i <- check_net_input( my_genes, network_list[['dPPI']] )
table(i)
names(i)[i == 'no']

## ----example3_1, echo=TRUE----------------------------------------------------
my_genes <- geneset_list$MSigDB_C7[['GOLDRATH_NAIVE_VS_MEMORY_CD8_TCELL_UP']]
net3.1 <- network_overlap( my_genes, resources = 'PID',
                           include_neighbors = FALSE, dedup = TRUE )

nets2use <- c('PID','dPPI','TFe','HumanNet','CCSB')
net3.2 <- network_overlap( my_genes, resources = nets2use,
                           include_neighbors = FALSE, dedup = TRUE )

net3.3 <- network_overlap( my_genes, resources = 'PID',
                           include_neighbors = TRUE, dedup = TRUE )

## ----example4, fig.height = 5, fig.width = 5, fig.align = 'center'------------
require(igraph)
net4 <- network_overlap( my_genes, resources = c('PID','dPPI','TFe'),
                         include_neighbors = FALSE, dedup = TRUE )
edges <- as.matrix( net4[, c(1,3)] )
G <- igraph::make_undirected_graph( c(t(edges)) )
par(mar=rep(0,4))
plot(G, vertex.size = 20, vertex.frame.color = 'white' )

## ----example5, fig.height = 5, fig.width = 5, fig.align = 'center'------------
require(igraph)

G <- as.graph( network_list$PID )
all( c("EP300", "SFN", "TP53", "CCNB1") %in% names(V(G)) )
get.shortest.paths(G, "TP53" , to="CCNB1", mode = "all" )$vpath
get.shortest.paths(G, "EP300", to="SFN"  , mode = "all" )$vpath

G <- as.graph( network_list$HumanNet )
all( c("EP300", "SFN", "TP53", "CCNB1") %in% names(V(G)) )
get.shortest.paths(G, "TP53" , to="CCNB1", mode = "all" )$vpath
get.shortest.paths(G, "EP300", to="SFN"  , mode = "all" )$vpath

## ----example6, echo=TRUE, eval=FALSE------------------------------------------
# my_genes <- geneset_list$MSigDB_C2[['VERNOCHET_ADIPOGENESIS']]
# net5 <- network_overlap( my_genes )
#  g <- unique(c( net5$p1, net5$p2 ))
# 
# tab <- data.frame( gene      = c('FABP4',  'CEBPA','PPARG','ADRB3','RETN','AGT','HP',
#                                  'RARRES2','PANK3','FFAR2','LUM',  'MC2R','ADCYAP1R1'),
#                    TrogRatio = c( 1.8,      1.7,    0.6,    0.3 ,   0.3 ,  0.4  ,0.2,
#                                   0.3,      0.1,    0.5,    0.3 ,   0.5 ,  0.1),
#                    WAT_BAT   = c( 0.8,      1.0,    0.6,    10.0,   21.6,  215.4,2.4,
#                                   9.5,      3.9,    4.6,    4.0 ,   7.3 ,  2.6),
#                    initial   = g %in% my_genes
#                    )
# 
# write_simple_table(net3.1, 'net_example.sif')
# write_simple_table(tab,    'net_example.tab')

## ----example7, echo=TRUE, eval=FALSE------------------------------------------
# 
# ## Add a new resource to "network_list"
# ### For brevity, we
# network_list[['BioGRID_Mouse']] <- readSIF( 'BIOGRID-ORGANISM-Mus_musculus-3.4.136.symbols.sif.gz', header = TRUE )
# # > str(network_list[['BioGRID_Mouse']])
# # 'data.frame':	38322 obs. of  3 variables:
# #  $ p1       : chr  "SMAD2" "SMAD2" "SMAD2" "SMAD2" ...
# #  $ edge_type: chr  "physical" "physical" "physical" "physical" ...
# #  $ p2       : chr  "Rasd2" "Rab34" "Rhebl1" "Rab38" ...
# 
# ## Short example from Tang's 2010 Nature paper
# my_mouse <- c('Sost','Fxyd4','Tmprss6','Crtap','Thpo','Kcnn4','Osm','Slc29a3','ALB')
# 
# ## First, check if these genes appear in the BioGRID network.
# check_net_input( my_mouse, network_list[['BioGRID_Mouse']] )
# #  Sost   Fxyd4 Tmprss6   Crtap    Thpo   Kcnn4     Osm Slc29a3     ALB
# # "yes"    "no"    "no"    "no"    "no"    "no"    "no"    "no"    "no"
# 
# ## After correcting a few gene names, get the induced subnetwork from mouse data.
# my_mouse <- c('Sost','Fxyd4','Tmprss6','CRTAP','Thpo','KCNN4','Osm','Slc29a3','ALB')
# net.m <- network_overlap( my_mouse, include_neighbors = TRUE, resources = c('BioGRID_Mouse') )
# str(net.m)
# # Generating undirected subnetwork...
# # Total induced subnetwork from 9 genes has 17 nodes and 17 edges (17 unique).
# # 'data.frame':	17 obs. of  3 variables:
# #  $ p1       : chr  "Sf3a1" "Nphp1" "Iqcb1" "Invs" ...
# #  $ edge_type: chr  "physical" "physical" "physical" "physical" ...
# #  $ p2       : chr  "CRTAP" "Invs" "Nphp1" "ALB" ...
# 
# ## Also, check within BioGRD's human network
# check_net_input( my_mouse, network_list[['BioGRID_Human']] )
# # Sost   Fxyd4 Tmprss6   CRTAP    Thpo   KCNN4     Osm Slc29a3     ALB
# # "no"    "no"    "no"   "yes"    "no"   "yes"    "no"    "no"   "yes"
# 
# ## Note that gene symbols are case sensitive
# my_mouse <- c('SOST','Fxyd4','Tmprss6','CRTAP','THPO','KCNN4','OSM','Slc29a3','ALB')
# check_net_input( my_mouse, network_list[['BioGRID_Human']] )
# #  SOST   Fxyd4 Tmprss6   CRTAP    THPO   KCNN4     OSM Slc29a3     ALB
# # "yes"    "no"    "no"   "yes"    "no"   "yes"   "yes"    "no"   "yes"
# 
# ## Get the induced subnetowrk from human data
# net.h <- network_overlap( my_mouse, include_neighbors = TRUE, resources = c('BioGRID_Human') )
# str(net.h)
# # Generating undirected subnetwork...
# # Total induced subnetwork from 9 genes has 224 nodes and 755 edges (634 unique).
# # 'data.frame':	755 obs. of  3 variables:
# #  $ p1       : chr  "MBIP" "SH3GL1" "TNNT1" "GFAP" ...
# #  $ edge_type: chr  "physical" "physical" "physical" "physical" ...
# #  $ p2       : chr  "MBIP" "SH3GL1" "TNNT1" "GRAP2" ...
# 

## ----example8, echo=TRUE, eval=TRUE-------------------------------------------
net <- network_overlap( 'FOXP3', include_neighbors = TRUE, resources = c("PID","dPPI","CCSB" ) )
genes <- unique(c( net$p1, net$p2 ))
e1 <- term_enrichment( genes, "Blood_Translaiton_Modules", verbose=FALSE, all_symbols = cached_coding_genes )
summary(e1)
e2 <- term_enrichment( genes, "ReactomePathways", verbose=FALSE, all_symbols = cached_coding_genes )
summary(e2)

---
title: "Choosing Resources"
author: "Michael T. Zimmermann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Choosing Resources}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

RITAN indexes multiple resources and choosing which of them are most relevant for your study can be a challenge. To help with this process, we provide below a set of examples from different types of studies and the thinking behind which resources were used.

# Resources Indexed by default

```{r first_load, echo=TRUE, warning=FALSE, results='hide', message=FALSE}
library(RITANdata)
library(RITAN)
library(knitr)
kable( attr(network_list, 'network_data_sources') )
kable( sapply(geneset_list, length), col.names = c('# Genesets') )
```

While many resources contain information about protein complexes (obligate interactions) and protein-protein interactions (often transient), some use experimental techniques that are specific for physical ineractions. Determining whihc resources indicate physical, through-metabolite, and through-DNA (i.e. transcription factors) interactions, we recommend:
1) each resource's primary publication
2) the [pathguide](http://www.pathguide.org/) website
3) the following guidelines

# Example: Protein Complexes & Protein-Protein Interaction (PPI)

```{r geneset_overlap_P, echo=TRUE, eval=FALSE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'}
genes <- geneset_list$MSigDB_C5$APICAL_JUNCTION_COMPLEX
e <- network_overlap( genes, resources = c('CCSB','STRING'), minStringScore = 700 )
## We also strongly encourage use of the BioPlex database, which we do not distribute with RITAN in compliance with their licensing.
```

# Example: Metabolic Network Interaction

```{r geneset_overlap_M, echo=TRUE, eval=FALSE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'}
genes2 <- geneset_list$MSigDB_C5$AMINE_METABOLIC_PROCESS
e2 <- network_overlap( genes, resources = c('PID','HumanNet') )
```

# Example: Expression Signalling

```{r geneset_overlap_E, echo=TRUE, eval=FALSE, warning=FALSE, fig.width = 7, fig.height = 7, fig.align='center'}
genes2 <- geneset_list$MSigDB_C5$AMINE_METABOLIC_PROCESS
e2 <- network_overlap( genes, resources = c('ChEA','HumanNet') )
```

---
title: "Network Biology (Induced Subnetwork) Vignette"
author: "Michael T. Zimmermann"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Network Biology Vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{knitr}
---

<h1>Introduction</h1>
RITAN is a R package intended for Rapid Integration of Term Annotation and Network resources to be applied to gene sets generated by current analysis methods.

For a general introduction to the package, please see the "enrichment" vignette.

```{r load_data from package, echo=TRUE, results='hide', message=FALSE}
library(RITANdata)
library(RITAN)
require(knitr)
```

The R object, *network_list*, contains `r length(network_list)` human network-biology resources. Additionally, RITAN leverages existing R packages to access data from STRING.
[In 08-2018, ProNet was depricated and internal support for HPRD and BioGRID using ProNet APIs was lost. We have added to our recommended setup script, instructions for adding both of these resources to RITAN.].
Citation information for each resource can be accessed via attr():
```{r citation, echo=TRUE}
kable( attr(network_list, 'network_data_sources') )
```

*NOTE that many of these resources are themselves combinations of other more specialized resources.

###We provide simple access to multiple network biology resources in order to facilitate rapid evaluation, basic data annotation, and integration using a common framework.

###We will elaborate on this functionality in the following examples.

------

#Example 1 - Basic Network Query

We begin by gathering a starting list of gene symbols. In this example, we choose the genes that are most highly up-regulated in human PBMCs, compared to healthy controls, gathered from an MSigDB module. See our "enrichment" vignette for further introduction to MSigDB. This example, named *my_genes* below, is intended to represent the outcome of an analysis - the genes identified by a high-throughput experiment. RITAN facilitates the integration of multiple network biology resources to query the know interactions within *my_genes*.


```{r example1, echo=TRUE}
my_genes <- geneset_list$MSigDB_C7[['GSE6269_HEALTHY_VS_FLU_INF_PBMC_DN']]
net <- network_overlap( my_genes, resources = 'CCSB' )
```

We now have what is referred to as the "induced subnetwork" for *my_genes* using only one resoure, CCSB. The network is the subset of interactions from CCSB that contains exclusively gene symbols within *my_genes*.

The input list contained `r length(unique(my_genes))` symbols and `r length(unique(c(net$p1,net$p2)))` were identified as having interactions within the network resources. Note: If very few genes have known connections between them, there may be a mismatch between your input format and the network resources - see the "Check Input" section below.

There are `r dim(net)[1]` edges returned by this query with the first 5 shown for example:
```{r example1.1, echo=TRUE}
head(unique(net))
```

The output format is a data.frame with three columns: 

Name  | Description
----- | -----------
p1    | The first interacting protein
edge_type | The manner in which p1 and p2 interact
p2    | The second interacting protein

The edge type is taken from the network resource, when possible. For instance, the first edge shown above indicates the self-reaction of the gene WAS. The second edge indicates protein-protein interaction between WAS and GRB2.


------

#Example 2 - Filtering out low-confidence interactions

Some network resources provide confidence scores for each interaction. Most notably, the popular STRING resource has indexed many other resources and additionally included computationally inferred interactions. Thus, many of the interactions within STRING have not been experimentally validated. While that does not mean they are incorrect, depending on your application you may not be comfortable with including them. We provide a simple means for trimming resources that provide scores using a simple threshold.

```{r example2, echo=TRUE, eval=FALSE}
net2 <- network_overlap( my_genes, resources = c('CCSB','STRING'), minStringScore = 700 )
str(net2)
```

Adding STRING brings the network to 86 genes and 265 edges among them. Thus, considering additional resources increases the coverage of the input set of genes. 

Scores within STRING are on a [0, 1000] scale with 1000 taken as "fully confident." We recommend using a threshold of 700 (the default) for "high confidence" investigation. The further integration of multiple resources (as performed here) will help to achieve breadth of coverage without resorting to inclusion of low-confidence interactions.

RITAN currently indexes `r length(network_list)+3` resources which can be quiried individually, or together to achieve greater coverage. As each resource is derived from different types of experiments, consider if the source and interaction type are applicable to your dataset. For a reference listing for resources, see attr(network_list, 'network_data_sources').

------

#Check Input

###Check how many input gene symbols are within the network resources.

A good quality-control step is to check how many of your input genes of interest actually appear in any of the network resources. If too few appear, you either have identified a set of poorly understood genes (novel finding), or the symbols used do not match (gene symbol version mismatch). The second is more likely.

To quickly check if your symbols appear in any of the loaded network resources:
```{r check_input1, echo = TRUE }
my_genes <- geneset_list$MSigDB_C2[['VERNOCHET_ADIPOGENESIS']]
i <- check_any_net_input( my_genes )
table(i)
```

From the above, you can see that all (save one) genes in the adipogenesis geneset appear in a network resource. Thus, we can be confident that that the combined network resource well-represents our genes of interest. Additionally, the gene symbols in use by our input study (in this case, MSigDB) and RITAN, are compatible.

Users may want to only use one or a few selected resources. This may be useful for many reasons including comparison to previous literature or preference for direct physical interactions over regulatory interactions. To perform the analogous check using one resource:
```{r check_input2, echo = TRUE }
i <- check_net_input( my_genes, network_list[['dPPI']] )
table(i)
names(i)[i == 'no']
```

Now, it is evident that while across the resources indexed by RITAN, all of the genes are represented, the representation by any one resource may be less comprehensive. This is the primary reason that we have provided multiple resources within RITAN - to achieve more comprehensive coverage.


------

#Example 3 - Including Neighbors

In many instances, the known relationships within a group of genes is the most direct (as above). However, a broader analysis considering what other genes interact with the input list may be informative. If only a few top candidates are of interest, prioritized by computational methods, we suggest established tools such as GeneMANIA (webserver or Cytoscape App) or CyTargetLinker. RITAN will provide all neighbors, with stringency filtering provided.

Because the previous example has a large number of genes, we will switch to a smaller example. Again leveraging MSigDB, we gather an example of `r length(geneset_list$MSigDB_C2[['VERNOCHET_ADIPOGENESIS']])` genes "up-regulated with adipogenic differentiation and down-regulated by troglitazone."

```{r example3_1, echo=TRUE}
my_genes <- geneset_list$MSigDB_C7[['GOLDRATH_NAIVE_VS_MEMORY_CD8_TCELL_UP']]
net3.1 <- network_overlap( my_genes, resources = 'PID',
                           include_neighbors = FALSE, dedup = TRUE )

nets2use <- c('PID','dPPI','TFe','HumanNet','CCSB')
net3.2 <- network_overlap( my_genes, resources = nets2use,
                           include_neighbors = FALSE, dedup = TRUE )

net3.3 <- network_overlap( my_genes, resources = 'PID',
                           include_neighbors = TRUE, dedup = TRUE )
```

First, we identified the induced subnetwork about our geneset of interest within the Pathway Interaction Database (PID). We added the *dedup* option to remove duplicate edges.

Second, we augmented this subnetwork by using multiple network resources. As in the previous examples, the coverage of the input geneset is markedly increased.

Third, we augmented the PID-only network by including all of the first-neighbors of input genes. The "small-world" and context-non-specific nature of these network resources can easily lead to massive networks with thousands of genes included, making interpretation very difficult (a.k.a. unwieldy "hairballs"). Thus, we recommend pre-planned and thought-out analysis plans, when possible, to focus your network-based study on the most relevant interactions.

The neighbors included in lists such as net3.3 indicate candidates for further study that could modulate the activity of, in this case, differentially expressed genes.

------

#Example 4 - Visualize Network within R

RITAN facilitates generating or gathering a network, while existing methods exist for analysis of the network. One solution is the igraph package which provides a wide range of network analysis and visualization functionality. For brevity, we will not query the larger network resources here.

From the following example plot, we can identify HIF1-alpha as a hub gene within the Memory-CD8-T-cell up-regulated genes loaded in the previous example.

```{r example4, fig.height = 5, fig.width = 5, fig.align = 'center' }
require(igraph)
net4 <- network_overlap( my_genes, resources = c('PID','dPPI','TFe'),
                         include_neighbors = FALSE, dedup = TRUE )
edges <- as.matrix( net4[, c(1,3)] )
G <- igraph::make_undirected_graph( c(t(edges)) )
par(mar=rep(0,4))
plot(G, vertex.size = 20, vertex.frame.color = 'white' )
```


------

#Example 5 - Identify "Bridging" Genes/Proteins

RITAN is a generic tool that can be used with other tools to facilitate novel analyses.

Say you have two genes of interest that do not interact with one another, but you suspect that they influence one another. You can use the igraph package with RITAN to investigate this question.

Below, we show how to identify “bridging” genes/proteins from a network using the example of two cell cycle checkpoint proteins, TP53 and SFN.

```{r example5, fig.height = 5, fig.width = 5, fig.align = 'center' }
require(igraph)

G <- as.graph( network_list$PID )
all( c("EP300", "SFN", "TP53", "CCNB1") %in% names(V(G)) )
get.shortest.paths(G, "TP53" , to="CCNB1", mode = "all" )$vpath
get.shortest.paths(G, "EP300", to="SFN"  , mode = "all" )$vpath

G <- as.graph( network_list$HumanNet )
all( c("EP300", "SFN", "TP53", "CCNB1") %in% names(V(G)) )
get.shortest.paths(G, "TP53" , to="CCNB1", mode = "all" )$vpath
get.shortest.paths(G, "EP300", to="SFN"  , mode = "all" )$vpath
```


------

#Example 6 - Write data for import into Cytoscape

Cytoscape is a powerful platform for network analysis that has gathered a large following from divers fields of application. Thus, many extensions (Apps) have been written for Cytoscape. RITAN facilitates the generation of networks using integrated resources and those networks can be exported for use in Cytoscape. Beginning from one of the induced subnetworks generated above, we demonstrate export of this data to simple text files and how it can be imported into Cytoscape for visualization and presentation.

The publication for the geneset is available  [here](http://www.ncbi.nlm.nih.gov/pmc/articles/PMC2725706/) and describes which genes are "induced by white adipocyte differentiation and inhibited by troglitazone activation of PPAR-gamma." Taking data from Table 1, we annotate the genes by functional data. Additionally, we add a variable to indicate which genes are within the initial list of genes.
```{r example6, echo=TRUE, eval=FALSE }
my_genes <- geneset_list$MSigDB_C2[['VERNOCHET_ADIPOGENESIS']]
net5 <- network_overlap( my_genes )
 g <- unique(c( net5$p1, net5$p2 ))

tab <- data.frame( gene      = c('FABP4',  'CEBPA','PPARG','ADRB3','RETN','AGT','HP',
                                 'RARRES2','PANK3','FFAR2','LUM',  'MC2R','ADCYAP1R1'), 
                   TrogRatio = c( 1.8,      1.7,    0.6,    0.3 ,   0.3 ,  0.4  ,0.2, 
                                  0.3,      0.1,    0.5,    0.3 ,   0.5 ,  0.1),
                   WAT_BAT   = c( 0.8,      1.0,    0.6,    10.0,   21.6,  215.4,2.4, 
                                  9.5,      3.9,    4.6,    4.0 ,   7.3 ,  2.6),
                   initial   = g %in% my_genes
                   )

write_simple_table(net3.1, 'net_example.sif') 
write_simple_table(tab,    'net_example.tab')
```

The `write_simple_table` will produce plain text files for input into Cytoscape.

**Briefly**, these can be imported into Cytoscape by:

- Import the network with: File --> Import --> Network --> File...

- Import the data table with: File --> Import --> Table --> File...

**See the [Cytoscape Tutorial](wiki.cytoscape.org/GettingStarted)** for thorough examples of importing data into Cytoscape.
<!-- *We will provide an introduction to Cytoscape and how to use data imported from RITAN. -->
<!-- **A related introduction is available [here](https://www.youtube.com/watch?v=-3VSMJCHrV8); the process for RITAN data will be very similar. -->



------

#Example 7 - Use Your Own Network Resource

For many reasons, you may wish to use a network resource that we have not provided with this package. Some examples include use of a different ID system, non-human data, or use of a gene coexpression network. In this example, we use a short example of genes with tissue-specific expression from [Tang's 2010 Nature paper](http://www.nature.com/nbt/journal/v28/n7/pdf/nbt.1644.pdf), and the mouse interaction network from BioGRID.

```{r example7, echo=TRUE, eval=FALSE}

## Add a new resource to "network_list"
### For brevity, we
network_list[['BioGRID_Mouse']] <- readSIF( 'BIOGRID-ORGANISM-Mus_musculus-3.4.136.symbols.sif.gz', header = TRUE )
# > str(network_list[['BioGRID_Mouse']])
# 'data.frame':	38322 obs. of  3 variables:
#  $ p1       : chr  "SMAD2" "SMAD2" "SMAD2" "SMAD2" ...
#  $ edge_type: chr  "physical" "physical" "physical" "physical" ...
#  $ p2       : chr  "Rasd2" "Rab34" "Rhebl1" "Rab38" ...

## Short example from Tang's 2010 Nature paper
my_mouse <- c('Sost','Fxyd4','Tmprss6','Crtap','Thpo','Kcnn4','Osm','Slc29a3','ALB')

## First, check if these genes appear in the BioGRID network.
check_net_input( my_mouse, network_list[['BioGRID_Mouse']] )
#  Sost   Fxyd4 Tmprss6   Crtap    Thpo   Kcnn4     Osm Slc29a3     ALB
# "yes"    "no"    "no"    "no"    "no"    "no"    "no"    "no"    "no"

## After correcting a few gene names, get the induced subnetwork from mouse data.
my_mouse <- c('Sost','Fxyd4','Tmprss6','CRTAP','Thpo','KCNN4','Osm','Slc29a3','ALB')
net.m <- network_overlap( my_mouse, include_neighbors = TRUE, resources = c('BioGRID_Mouse') )
str(net.m)
# Generating undirected subnetwork...
# Total induced subnetwork from 9 genes has 17 nodes and 17 edges (17 unique).
# 'data.frame':	17 obs. of  3 variables:
#  $ p1       : chr  "Sf3a1" "Nphp1" "Iqcb1" "Invs" ...
#  $ edge_type: chr  "physical" "physical" "physical" "physical" ...
#  $ p2       : chr  "CRTAP" "Invs" "Nphp1" "ALB" ...

## Also, check within BioGRD's human network
check_net_input( my_mouse, network_list[['BioGRID_Human']] )
# Sost   Fxyd4 Tmprss6   CRTAP    Thpo   KCNN4     Osm Slc29a3     ALB
# "no"    "no"    "no"   "yes"    "no"   "yes"    "no"    "no"   "yes"

## Note that gene symbols are case sensitive
my_mouse <- c('SOST','Fxyd4','Tmprss6','CRTAP','THPO','KCNN4','OSM','Slc29a3','ALB')
check_net_input( my_mouse, network_list[['BioGRID_Human']] )
#  SOST   Fxyd4 Tmprss6   CRTAP    THPO   KCNN4     OSM Slc29a3     ALB 
# "yes"    "no"    "no"   "yes"    "no"   "yes"   "yes"    "no"   "yes"

## Get the induced subnetowrk from human data
net.h <- network_overlap( my_mouse, include_neighbors = TRUE, resources = c('BioGRID_Human') )
str(net.h)
# Generating undirected subnetwork...
# Total induced subnetwork from 9 genes has 224 nodes and 755 edges (634 unique).
# 'data.frame':	755 obs. of  3 variables:
#  $ p1       : chr  "MBIP" "SH3GL1" "TNNT1" "GFAP" ...
#  $ edge_type: chr  "physical" "physical" "physical" "physical" ...
#  $ p2       : chr  "MBIP" "SH3GL1" "TNNT1" "GRAP2" ...

```

From comparison of the above results, there are noticeable differences between using each resource. Some genes are present in the mouse interaction network that are not present in the human (e.g. Fxyd4). Also, there is usually a greater extent of interaction data available for human genes/proteins. This may be more helpful for some studies, but too indirect for others.

------

#Example 8 - Putting it All Together

**RITAN's two primary functions can be used together in order to gain stronger insights into the functional role of the identified gene(s).**

Between the two RITAN vignettes, you've seen how to use RITAN to perform term enrichment and to generate subnetworks leveraging multiple data sources. In the following example, we will start from a single gene of interest, gather the genes that it is known to physically interact with (protein-protein interaction; PPI), and then perform enrichment analysis on group of interacting genes/proteins.

```{r example8, echo=TRUE, eval=TRUE }
net <- network_overlap( 'FOXP3', include_neighbors = TRUE, resources = c("PID","dPPI","CCSB" ) )
genes <- unique(c( net$p1, net$p2 ))
e1 <- term_enrichment( genes, "Blood_Translaiton_Modules", verbose=FALSE, all_symbols = cached_coding_genes )
summary(e1)
e2 <- term_enrichment( genes, "ReactomePathways", verbose=FALSE, all_symbols = cached_coding_genes )
summary(e2)
```

Thus, starting from only one gene of interest, we identified 9 additional physically interacting genes. These genes are known to be co-regulated in a Blood-Translation-Module associated with T-cell signaling. Additionally, they are significantly associated with four Reactome pathways including AP-1 and MAPK signaling

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{summary.term_enrichment_by_subset}
\alias{summary.term_enrichment_by_subset}
\title{summary.term_enrichment_by_subset}
\usage{
\method{summary}{term_enrichment_by_subset}(object, verbose = TRUE, ...)
}
\arguments{
\item{object}{data frame returned by term_enrichment_by_subset()}

\item{verbose}{if TRUE (default), print a header describing the data type}

\item{...}{Further arguments are passed on to head()}
}
\value{
the data.frame of top enrichment results
}
\description{
summary.term_enrichment_by_subset
}
\examples{
require(RITANdata)
vac1.de.genes <- list(vac1.day0vs31.de.genes, vac1.day0vs56.de.genes)
names(vac1.de.genes) <- c("Day0vs31", "Day0vs56")
e <- term_enrichment_by_subset(vac1.de.genes, "MSigDB_Hallmarks", q_value_threshold = 0.1 )
summary(e)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{writeGMT}
\alias{writeGMT}
\title{writeGMT}
\usage{
writeGMT(s, file = NA, link = rep("", length(s)))
}
\arguments{
\item{s}{list of gene sets in current R session. Each entry will become a row in the GMT file.}

\item{file}{file name to write to}

\item{link}{default is "". This is the second column of a GMT file and is usually a hyperlink or note about the origin of the term}
}
\value{
Nothing is returned. A file is written.
}
\description{
Created for future use and simplification of writing .gmt files from the package.
}
\examples{
# Make an example list() to show the GMT format
set <- list( term1=c('gene_name1','gene_name2'),
             term2=c('gene_name3','gene_name4','gene_name5') )
\dontrun{
# Write a GMT file for "set"
writeGMT( set, 'my_file.gmt')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\docType{data}
\name{vac2.day0vs31.de.genes}
\alias{vac2.day0vs31.de.genes}
\title{This dataset is included as an example in the package:}
\format{
An object of class \code{character} of length 522.
}
\usage{
vac2.day0vs31.de.genes
}
\value{
differentially expressed genes at 31 days post-vaccination with vaccine2
}
\description{
This dataset is included as an example in the package:
}
\examples{
\dontrun{
 #data("vac2.day0vs31.de.genes")
 te <- term_enrichment(geneset = vac2.day0vs31.de.genes)
}

}
\references{
\url{https://www.ncbi.nlm.nih.gov/pubmed/26755593}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{plot.term_enrichment}
\alias{plot.term_enrichment}
\title{plot.term_enrichment}
\usage{
\method{plot}{term_enrichment}(x = NA, min_q = 0.05, max_terms = 25, extend_mar = c(0, 10, 0, 0), ...)
}
\arguments{
\item{x}{data frame returned by term_enrichment}

\item{min_q}{Only q-values more significant than this threshold will be plotted. Default = 0.05.}

\item{max_terms}{Up to max_terms will be plotted. Default = 25.}

\item{extend_mar}{Term names can be long. We attempt to keep them readable by extending the left-hand-side margins automatically. Default = c(0,10,0,0) added to par()$mar.}

\item{...}{Additional arguments are passed on to plot()}
}
\value{
silent return from plot
}
\description{
plot.term_enrichment
}
\examples{
require(RITANdata)
e <- term_enrichment(vac1.day0vs31.de.genes, resources = 'GO_slim_generic')
plot(e, min_q = .1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{geneset_overlap}
\alias{geneset_overlap}
\title{geneset_overlap}
\usage{
geneset_overlap(s1, s2 = s1, s.size = unlist(lapply(s1, length)))
}
\arguments{
\item{s1}{The first geneset}

\item{s2}{the second geneset}

\item{s.size}{Denominator used in each comparison. The default is to determint the lengths of elements in "s1"}
}
\value{
results matrix of input gene list compared to active gene sets. Q value is calculated using entire group of active gene sets.
}
\description{
Return assymetric matrix of the fraction of genes shared between sets. E.G. The fraction of the first set that is "covered" by or "overlaps" the second set.
}
\examples{
require(RITANdata)
r <- geneset_overlap( geneset_list$MSigDB_Hallmarks, geneset_list$NetPath_Gene_regulation )
heatmap(r, col = rev(gray(seq(0,1,length.out = 15))) )
summary(c(r))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{enrichment_symbols}
\alias{enrichment_symbols}
\title{enrichment_symbols}
\usage{
enrichment_symbols(geneset, term = NULL, all_symbols = NA, ...)
}
\arguments{
\item{geneset}{vector of gene symbols to be evaluated}

\item{term}{a list containing specific gene set term(s) and their corresponding gene symbols contained in one of the annotation resources, default is all gene set terms
in the GO, ReactomePathways, KEGG_filtered_canonical_pathways, and MSigDB_Hallmarks libraries}

\item{all_symbols}{gene symbols to be evaluated, identified by gene symbol name. Default is all protein coding genes. This parameter should be manipulated to include only the gene symbols that pertain to the user's analysis.}

\item{...}{additional arguments are not used}
}
\value{
results matrix of input gene list compared to active gene sets. Q value is calculated using entire group of active gene sets.
}
\description{
This function is called by term_enrichment() and term_enrichment_by_subset(). The user may call it directly, but we suggest using term_enrichment(). The function uses the resources currently loaded into the active_genesets vector. See load_geneset_symbols().
}
\details{
Outputs a data frame containing the gene set name, a hypergeometric-test p value, the number of genes from the
input gene list that occur in the gene set, the number of genes in the gene set, the gene symbols for
the genes in the input gene list, and the q value.
}
\examples{
require(RITANdata)
myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')

\dontrun{
## We suggest using term_enrichment() instead. E.g.:
e <- term_enrichment(myGeneSet, 'GO')
}

## But, you may use enrichment_symbols() directly for an individual term:
load_geneset_symbols('GO')
e <- enrichment_symbols(myGeneSet, 'DNA_repair', all_symbols = cached_coding_genes)
print(e)

\dontrun{
## Gene set enrichment using intersection of gene symbols 
##   provided in geneset parameter and all protein coding genes.
enrichment_symbols(geneset = vac1.day0vs31.de.genes)

## choose which terms to evaluate
t <- active_genesets[1:5]

## Test enrichment of that set of terms
enrichment_symbols(geneset = vac1.day0vs31.de.genes, term = t)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interconnectivity_functions.R
\name{icon_test}
\alias{icon_test}
\title{icon_test}
\usage{
icon_test(nodes1 = NULL, nodes2 = NULL, s = 100, verbose = TRUE, ...)
}
\arguments{
\item{nodes1}{[NULL] the first network. See network_overlap().}

\item{nodes2}{[NULL] the second network. See network_overlap().}

\item{s}{[100] the number of random permutations to make.}

\item{verbose}{[TRUE] Extent of text shown in the console.}

\item{...}{Additional argumetns are passed on to the specific test performed}
}
\value{
metrics and significance of the network overlap
}
\description{
"icon" is an abbreviation for the "interconnectivity" of a network or graph.
}
\details{
This function handles different inputs and directs them to the appropriate "icon" testing method.
Depending on the values given to "nodes1" and "nodes2," a different specific test is performed.

Note that the specific functions called make use of the "param" attribute of each input.
These parameters are populated by network_overlap() so that the permutation reflects the exact procedure that was done to generate "nodes1" and/or "nodes2."
}
\examples{
\dontrun{
icon_test( nodes1=n, s=10)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interconnectivity_functions.R
\name{icon_single_within}
\alias{icon_single_within}
\title{icon_single_within
interconnectivity score within a network}
\usage{
icon_single_within(nodes = NULL, net = NULL, s = 10, verbose = TRUE)
}
\arguments{
\item{nodes}{the node labels to use}

\item{net}{the network to use}

\item{s}{[10] the number of repeated random draws to make}

\item{verbose}{[TRUE] if more verbose output should be shown}
}
\description{
icon_single_within
interconnectivity score within a network
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_network.R
\name{as.graph}
\alias{as.graph}
\title{as.graph}
\usage{
as.graph(mat, p1 = 1, p2 = 3, ...)
}
\arguments{
\item{mat}{matrix or data frame describing a network}

\item{p1}{[1] column of first interactor}

\item{p2}{[3] column of second interactor}

\item{...}{further options passed on to igraph::graph()}
}
\value{
igraph object
}
\description{
wrapper to convert a data.frame from RITAN an igraph graph object
}
\examples{
\dontrun{
G <- as.graph(network_list$PID)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\docType{data}
\name{vac1.day0vs56.de.genes}
\alias{vac1.day0vs56.de.genes}
\title{This dataset is included as an example in the package:}
\format{
An object of class \code{character} of length 471.
}
\usage{
vac1.day0vs56.de.genes
}
\value{
differentially expressed genes at 56 days post-vaccination with vaccine1
}
\description{
This dataset is included as an example in the package:
}
\examples{
\dontrun{
 #data("vac1.day0vs56.de.genes")
 te <- term_enrichment(geneset = vac1.day0vs56.de.genes)
}

}
\references{
\url{https://www.ncbi.nlm.nih.gov/pubmed/26755593}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_network.R
\name{network_overlap}
\alias{network_overlap}
\title{network_overlap}
\usage{
network_overlap(
  gene_list = NA,
  resources = c("PID", "TFe", "dPPI", "CCSB", "STRING"),
  minStringScore = 700,
  minHumanNetScore = 0.4,
  minScore = 0,
  verbose = TRUE,
  dedup = TRUE,
  directed_net = FALSE,
  include_neighbors = FALSE,
  STRING_cache_directory = NA,
  STRING_species = 9606,
  STRING_version = "10"
)
}
\arguments{
\item{gene_list}{A list of genes to use. The function will identify edges across resources for or among these genes; identify the induced subnetwork around the gene_list.}

\item{resources}{Name of network resource(s) to use.}

\item{minStringScore}{If STRING is among the resources, only edges of at least the indicated score will be included.}

\item{minHumanNetScore}{If HumanNet is among the resources, only edges of at least the indicated score will be included.}

\item{minScore}{Same as above, but used for any other networks where "score" is provided}

\item{verbose}{If TRUE (default), the function will update the user on what it is doing and how many edges are identified for each resource.}

\item{dedup}{If TRUE (Default = TRUE), remove edges reported by multiple resources. The edge type will be a semi-colon delimited list of the resources that had reported the interaction.}

\item{directed_net}{Logical indicating if the network resources should be interpreted as directed.}

\item{include_neighbors}{Logical to include 1st neighbors of "gene_list" (genes not in gene_list, but directly connected to them) in the induced subnetwork.}

\item{STRING_cache_directory}{A directory where STRING data files are cached to speed up subsequent queries; no need to re-download. If NA (the default), caches STRING data in your Rpackages directory. If "", uses a temporary directory that is cleared when the R-session closes.}

\item{STRING_species}{Species taxon ID (number) to use in searching STRING data. (Default = 9606)}

\item{STRING_version}{Version of the STRING database (Default = "10")}
}
\value{
Data table describing the induced subnetwork for "gene_list" across the requested resources.
}
\description{
network_overlap
}
\examples{
## Get interactions among a list of genes from the PID: Pathway Interaction Database
require(RITANdata)
myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
sif <- network_overlap( myGeneSet, resources = 'PID')
print(sif)

\dontrun{
## Get the PPI network induced by genes within myGeneSet
## Use 4 separate resources, but trim STRING to only include more confident interactions
sif <- network_overlap( myGeneSet, c('dPPI','PID','CCSB','STRING'), minStringScore = 500 )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_network.R
\name{readSIF}
\alias{readSIF}
\title{readSIF}
\usage{
readSIF(
  file = NA,
  header = FALSE,
  sep = "\\t",
  as.is = TRUE,
  p1 = 1,
  p2 = 2,
  et = 3,
  score = NA,
  ...
)
}
\arguments{
\item{file}{location of file}

\item{header}{indicator of presense of header on file}

\item{sep}{file delimiter - used by read.table()}

\item{as.is}{logical (default TRUE)}

\item{p1}{Column number for the 1st entity. Default = 1.}

\item{p2}{Column number for the 2nd entity. Default = 2.}

\item{et}{Column number for the edge type. Default = 3. Optionally, it may be a string label to be used as the edge type for all interactions from the input file.}

\item{score}{Column number for edge scores or weights. Default = NA (no score read).}

\item{...}{Other options to read.table().}
}
\value{
Returns a data.frame with 3 (or 4) columns of data.
}
\description{
This function reads a data table into R; the data table describes network interactions. It is named for the Simple Interaction Format (SIF), but can read any data table if the users identifies which columns contain the pertinent data (see below).
}
\details{
The SIF file format is a 3-column format, with an optional 4th column:
<entity-1><tab><edge-type><tab><entity-2><tab><score>

Entities may be genes, proteins, metabolites, etc. The edge type typically conveys the type of relationship that exists between the two entities, such as physical interaciton, phosphorylation, or activation.
}
\examples{
# Make a simple example to show the SIF file format
s <- matrix(c('gene1','gene2','PPI',
              'gene1','gene3','Chip-Seq',
              'gene4','gene5','PPI'), ncol=3, byrow=TRUE)
\dontrun{
# Read a SIF file
write.table( s, "myFile.sif", sep='\t', col.names=FALSE, row.names=FALSE )
sif <- readSIF("myFile.sif")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{resource_reduce}
\alias{resource_reduce}
\title{resource_reduce
Merge terms across resources to reduce the number of redundant and semi-redundant terms}
\usage{
resource_reduce(genesets = NULL, min_overlap = 0.8, verbose = TRUE)
}
\arguments{
\item{genesets}{the input genesets to consider. May be from one or multiple resources.}

\item{min_overlap}{terms that share at least this fraction of genes will be merged}

\item{verbose}{if TRUE, print status and summary output}
}
\value{
the list of terms, after merging to reduce redundnat and semi-redundant terms
}
\description{
resource_reduce
Merge terms across resources to reduce the number of redundant and semi-redundant terms
}
\examples{
require(RITANdata)
r <- resource_reduce( geneset_list$DisGeNet )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{show_active_genesets_hist}
\alias{show_active_genesets_hist}
\title{show_active_genesets_hist}
\usage{
show_active_genesets_hist(nbins = 50, ...)
}
\arguments{
\item{nbins}{Number of bins to include in histogram}

\item{...}{further argumants are passed on to plot()}
}
\value{
NULL. The plot is shown.
}
\description{
function to plot distribution of size of active_genesets object
}
\examples{
require(RITANdata)
load_geneset_symbols('GO_slim_generic')
show_active_genesets_hist()

\dontrun{
## Show the distribution of geneset sizes for the default set of geneset resources
load_geneset_symbols()
show_active_genesets_hist()

## Show the distribution of geneset sizes for a specific resource
load_geneset_symbols(gmt="ReactomePathways")
show_active_genesets_hist()

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{summary.term_enrichment}
\alias{summary.term_enrichment}
\title{summary.term_enrichment}
\usage{
\method{summary}{term_enrichment}(object, ...)
}
\arguments{
\item{object}{data frame returned by term_enrichment()}

\item{...}{Further arguments are passed on to head()}
}
\value{
the data.frame of top enrichment results
}
\description{
summary.term_enrichment
}
\examples{
require(RITANdata)
e <- term_enrichment( vac1.day0vs31.de.genes, "MSigDB_Hallmarks" )
summary(e, n=3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{readGMT}
\alias{readGMT}
\title{readGMT}
\usage{
readGMT(f = NA)
}
\arguments{
\item{f}{GMT file name. Please provide a full path if the file is not in the current working directory.}
}
\value{
A list() where the name of each entry is the term (first column of GMT file) and the value is a chr array of genes associated with the term.
}
\description{
Created for simplification of reading .gmt files into RITAN.
}
\examples{
# Make an example list() to show the GMT format
set <- list( term1=c('gene_name1','gene_name2'),
             term2=c('gene_name3','gene_name4','gene_name5') )
\dontrun{
# Write a GMT file for "set"
writeGMT( set, 'my_file.gmt' )

# Reading GMT files
geneset <- readGMT( 'my_file.gmt' )

# Additional GMT files are available from multiple sources.
#   We recommend:
#   http://software.broadinstitute.org/gsea/msigdb/
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interconnectivity_functions.R
\name{cov_undirected}
\alias{cov_undirected}
\title{cov_undirected
function to show the un-directed coverabe between two nodes lists from two networks}
\usage{
cov_undirected(this_nodes1, this_nodes2, this_net1, this_net2)
}
\arguments{
\item{this_nodes1}{list of nodes for first network}

\item{this_nodes2}{list of nodes for second network}

\item{this_net1}{the first network}

\item{this_net2}{the second network}
}
\description{
cov_undirected
function to show the un-directed coverabe between two nodes lists from two networks
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_network.R
\name{check_any_net_input}
\alias{check_any_net_input}
\title{check_any_net_input}
\usage{
check_any_net_input(set, resources = names(network_list))
}
\arguments{
\item{set}{An input list of genes to check against references.}

\item{resources}{The collection of network resources to check within.}
}
\value{
Logical vector indicating if the genes in "set" are within ANY of the resources.
}
\description{
A Quality Control function. This function applies check_net_input() to all available resources (default).
}
\examples{
#' ## Check if genes in myGeneSet are annotated by any resource in "network_list" (default).
library(RITANdata)
myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
yorn <- check_any_net_input( myGeneSet )
print(yorn)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{term_enrichment}
\alias{term_enrichment}
\title{term_enrichment}
\usage{
term_enrichment(
  geneset,
  resources = resources.default,
  report_resources_separately = FALSE,
  verbose = TRUE,
  all_symbols = NA,
  filter_to_intersection = FALSE,
  ...
)
}
\arguments{
\item{geneset}{vector of gene symbols to be evaluated}

\item{resources}{list containing the reference gene sets to test for enrichment}

\item{report_resources_separately}{logical (default FALSE) flag to report enrichments seperately for each requested resource, or to combine them and produce FDR adjustment across the combined set}

\item{verbose}{print the top results for each annotation resource}

\item{all_symbols}{the background/global set of gene symbols (study dependent; we provide all protien coding genes as a default)}

\item{filter_to_intersection}{[FALSE] should the background and foreground genesets be subsetted to one another?}

\item{...}{further arguments are passed on to enrichment_symbols()}
}
\value{
results matrix of input gene list compared to active gene sets. Q value is calculated within each of the active gene sets.
}
\description{
term_enrichment evaluates the input gene list for enrichment within each of the annotation resources. This differs from the enrichment_symbols function which evaluates the gene list for enrichment against all of the annotation resources grouped together.
}
\examples{
## Check if there is enrichment for any "Hallmark" functions within a input set of genes
require(RITANdata)
myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
e <- term_enrichment(myGeneSet, "MSigDB_Hallmarks")
print( e[1:2, -6] )

\dontrun{
term_enrichment(geneset = vac1.day0vs31.de.genes)
term_enrichment(geneset = vac1.day0vs31.de.genes, resources = "MSigDB_Hallmarks")
vac1.day0v31.enrichment <- term_enrichment(geneset = vac1.day0vs31.de.genes, verbose = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{term_enrichment_by_subset}
\alias{term_enrichment_by_subset}
\title{term_enrichment_by_subset}
\usage{
term_enrichment_by_subset(
  groups = NA,
  resources = resources.default,
  q_value_threshold = 0.01,
  verbose = TRUE,
  display_type = "q",
  phred = TRUE,
  ...
)
}
\arguments{
\item{groups}{A list() of genes for enrichment. Each entry in the list() is an input set of genes. Enrichment is performed for each of these entries.}

\item{resources}{character vector for which resources to use in enrichment}

\item{q_value_threshold}{minimum q-value (FDR adjusted p-value) in any group for the term to be included in results}

\item{verbose}{print additional status updates on what the function is doing}

\item{display_type}{Flag for which data type will be returned. One of "q" (default) for q-values, "p" for unadjusted p-values, or "n" for the number of genes overlapping the term.}

\item{phred}{Logical flag (default TRUE) to return the -log10 of p/q values}

\item{...}{Further arguments are passed on to enrichment_symbols()}
}
\value{
Returns a term-by-study matrix of enrichment values (value determined by "display_type")
}
\description{
Run enrichment simultaneously across a group of prioritized gene lists. For example, in a time course dataset, one may have a different list of genes that are differentially expressed at each time point. This function facilitates rapid evaluation of term enrichment across time point comparisons. Alternatively, one may have a different list of differentially expressed genes by drug treatment, environmental condition, ect.
}
\examples{
## Create list of gene sets to evaluate.
##   This example is from a vaccine study where we pre-generated differentially expressed genes.
##   This object will be passed to the groups parameter.
require(RITANdata)
vac1.de.genes <- list(vac1.day0vs31.de.genes, vac1.day0vs56.de.genes)
names(vac1.de.genes) <- c("Day0vs31", "Day0vs56")
print(str(vac1.de.genes))

\dontrun{
## Run term_enrichment_by_subset on the two results.
##   This function usually takes a few seconds to a minute to run.
m <- term_enrichment_by_subset(groups = vac1.de.genes, q_value_threshold = .9)
summary(m)
plot( m, label_size_y = 4, show_values = FALSE )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{load_geneset_symbols}
\alias{load_geneset_symbols}
\title{load_geneset_symbols}
\usage{
load_geneset_symbols(gmt = NA, gmt_dir = "", verbose = TRUE)
}
\arguments{
\item{gmt}{Either 1) name of pre-loaded resource (i.e. names(geneset_list)) or 2) gmt file containing annotation resources for enrichment annotation}

\item{gmt_dir}{location of gmt file named in gmt parameter}

\item{verbose}{print results to screen}
}
\value{
R list object named active_genesets
}
\description{
For most applications, this function is used internally by term_enrichment(). Users may call this function directly in some cases to force FDR adjustment to be across multiple resources. See Vignette for more details.
}
\details{
load_geneset_symbols allows the user to specify an annotation resource (e.g. Gene Ontology terms)
to use in enrichment analysis. The expectation is that the annotation resource contains of at least
one set of genes in the form of a list. The RITAN package comes with 15 pre-loaded annotation
resources. The default active annotation resources are GO, ReactomePathways,
KEGG_filtered_canonical_pathways, and MSigDB_Hallmarks.

The result of calling this function is to set the variable "active_genesets" which will be used by further functions.
}
\examples{
## Load generic GO-slim terms
require(RITANdata)
load_geneset_symbols("GO_slim_generic")
print(length(active_genesets))
print(head(active_genesets[[1]]))

\dontrun{
## load the default set of resources into "active_genesets"
load_geneset_symbols()

## Use only the Reactome Pathways annotation resource.
load_geneset_symbols(gmt="ReactomePathways")

## Suppresses output message describing the annotation resource and size.
load_geneset_symbols(gmt="ReactomePathways", verbose=FALSE)

## To list the available resources within RITAN:
print(names(geneset_list))

## You can also load your own data
load_geneset_symbols(gmt="myFile.gmt")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{load_all_protein_coding_symbols}
\alias{load_all_protein_coding_symbols}
\title{load_all_protein_coding_symbols}
\usage{
load_all_protein_coding_symbols()
}
\value{
A unique list of gene symbols for protein coding genes according to EnsDb.Hsapiens.v86
}
\description{
The character array returned is, by default, all human protein coding gene symbols. This variable defines the "universe of possible genes" for use in enrichment. Users should load a different "universe" or filter this one down to the most appropriate setting for their current study. For example, if running RNA-Seq, genes are in the universie if they are detected in any sample.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_network.R
\name{check_net_input}
\alias{check_net_input}
\title{check_net_input}
\usage{
check_net_input(
  set,
  ref,
  check4similar = FALSE,
  entity1name = "p1",
  entity2name = "p1"
)
}
\arguments{
\item{set}{An input list of genes to check against a reference.}

\item{ref}{A reference of network data. See readSIF().}

\item{check4similar}{Logical flag. If TRUE, a case-insensitive grep will be used for name matching. For genes in families with many related members (e.g. ABC*, FAM*, etc.), this will not be ideal. We intend this option as a QC screening method to identify if case, punctuaiton, etc is causing fewer than expected matches.}

\item{entity1name}{The column name in "ref" of the first entity. Default = "p1."}

\item{entity2name}{The column name in "ref" of the second entity. Default = "p2."}
}
\value{
Character vector of "yes/no" indicating "within-ref/not"
}
\description{
A Quality Control function. This function will compare an input list of genes to a network reference and report if each member of the input is present in the resource.
}
\examples{
## Return a "yes/no" vector indicating if each gene in myGeneSet is annotated with any term in GO
## If no match, this function can attempt to suggest closest matches (check4similar = TRUE)
library(RITANdata)
myGeneSet <- c('BRCA1','RAD51C','VAV1','HRAS','ABCC1','CYP1B1','CYP3A5')
yorn <- check_net_input( myGeneSet, network_list[["CCSB"]] )
print(yorn)

yorn <- check_net_input( myGeneSet, network_list[["PID"]] )
print(yorn)

## See check_any_net_input() for efficiently checking across all resources.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_enrichment.R
\name{plot.term_enrichment_by_subset}
\alias{plot.term_enrichment_by_subset}
\title{plot.term_enrichment_by_subset}
\usage{
\method{plot}{term_enrichment_by_subset}(
  x,
  show_values = TRUE,
  annotation_matrix = NA,
  low = "white",
  high = "#2166AC",
  return_ggplot_object = FALSE,
  label_size_x = 16,
  label_angle_x = -30,
  label_size_y = 9,
  wrap_y_labels = 20,
  grid_line_color = "white",
  mid = 0,
  cap = NA,
  annotation_palates = c("Reds", "Greens", "Purples", "Greys", "BuPu", "RdPu", "BrBG",
    "PiYG", "Spectral"),
  annotation_legend_x = -0.3,
  trim_resource_names = TRUE,
  ...
)
}
\arguments{
\item{x}{data frame returned by term_enrichment_by_subset}

\item{show_values}{True or False, plot values on the heatmap}

\item{annotation_matrix}{a matrix() of group-levle characteristics - same number of columns as "m"}

\item{low}{color for low end of range}

\item{high}{color for high end of range}

\item{return_ggplot_object}{logical flag (default FALSE) that if TRUE, the ggplot object for the plot is returned}

\item{label_size_x}{size of text for x label. Default lable_size_x=16}

\item{label_angle_x}{angle for text for x label. Default is -30 degrees}

\item{label_size_y}{size of text for y label. Default label_size_y=9}

\item{wrap_y_labels}{Number of characters to wrap row labels}

\item{grid_line_color}{color o grid lines between cells. Default is white.}

\item{mid}{sets lower threshold for color scale}

\item{cap}{Clip numeric values to this maximum threshold}

\item{annotation_palates}{Color palates (RColorBrewer) used for each row of the annotation matrix}

\item{annotation_legend_x}{offset for placing the legend}

\item{trim_resource_names}{[TRUE] remove any text in rownames preceeding a period characte. This convension is usually used in RITAN to prepend the resource name to the term name, which may not be needed in plotting.}

\item{...}{further areguments are not used at this time. If the user wants to modify the plot, use return_ggplot_object = TRUE.}
}
\value{
silent return, unless return_ggplot_object==TRUE. Then, the ggplot object for the plot is returned.
}
\description{
plot.term_enrichment_by_subset
}
\examples{
## Create list of gene sets to evaluate.
##   This example is from a vaccine study where we pre-generated differentially expressed genes.
##   This object will be passed to the groups parameter.
require(RITANdata)
vac1.de.genes <- list(vac1.day0vs31.de.genes, vac1.day0vs56.de.genes)
names(vac1.de.genes) <- c("Day0vs31", "Day0vs56")
print(str(vac1.de.genes))

\dontrun{
## Run term_enrichment_by_subset on the two results.
##   This function usually takes a few seconds to a minute to run.
m <- term_enrichment_by_subset(groups = vac1.de.genes, q_value_threshold = .9)
summary(m)
plot( m, label_size_y = 4, show_values = FALSE )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lib_network.R
\name{write_simple_table}
\alias{write_simple_table}
\title{write_simple_table}
\usage{
write_simple_table(d = NULL, f = NULL, ...)
}
\arguments{
\item{d}{R data object}

\item{f}{file path}

\item{...}{further options passed on to write.table}
}
\value{
invisible (nothing is returned)
}
\description{
This is a simple wrapper around "write.table" that writes a tab-delimited table with column names, no quoting, and no row names.
}
\examples{
\dontrun{
simple wrapper around write.table for writing a tab-delimieted, no row names, tab-seperated file
}
}
